@@ -38,10 +38,10 @@ public:
 
     static bool detect(const unsigned char* blob, size_t size);
 
-    virtual void read_file(const std::string& filepath);
-    virtual void read_stream(const char* content, size_t len);
+    virtual void read_file(const std::string& filepath) override;
+    virtual void read_stream(std::string_view stream) override;
 
-    virtual const char* get_name() const;
+    virtual std::string_view get_name() const override;
 
 private:
 
