@@ -11,11 +11,14 @@
 #include <cstdlib>
 #include <vector>
 #include <string>
-#include "pstring.hpp"
+#include <unordered_set>
 #include "env.hpp"
 
 namespace orcus {
 
+class xmlns_context;
+class xmlns_repository;
+
 /**
  * Generic constant to be used to indicate that a valid index value is
  * expected but not found.
@@ -24,28 +27,61 @@ ORCUS_PSR_DLLPUBLIC extern const size_t index_not_found;
 
 // XML specific types
 
-typedef size_t xml_token_t;
-typedef const char* xmlns_id_t;
+using xml_token_t = size_t;
+using xmlns_id_t = const char*;
+
+using xml_token_pair_t = std::pair<xmlns_id_t, xml_token_t>;
+
+struct ORCUS_PSR_DLLPUBLIC xml_token_pair_hash
+{
+    size_t operator()(const xml_token_pair_t& v) const;
+};
+
+using xml_elem_stack_t = std::vector<xml_token_pair_t>;
+using xml_elem_set_t = std::unordered_set<xml_token_pair_t, xml_token_pair_hash>;
 
 ORCUS_PSR_DLLPUBLIC extern const xmlns_id_t XMLNS_UNKNOWN_ID;
 ORCUS_PSR_DLLPUBLIC extern const xml_token_t XML_UNKNOWN_TOKEN;
 
-struct xml_name_t
+struct ORCUS_PSR_DLLPUBLIC parse_error_value_t
 {
-    xmlns_id_t ns;
-    pstring name;
+    std::string_view str;
+    std::ptrdiff_t offset;
 
-    xml_name_t() : ns(XMLNS_UNKNOWN_ID), name() {}
-    xml_name_t(xmlns_id_t _ns, const pstring& _name) : ns(_ns), name(_name) {}
-    xml_name_t(const xml_name_t& r) : ns(r.ns), name(r.name) {}
+    parse_error_value_t();
+    parse_error_value_t(std::string_view _str, std::ptrdiff_t _offset);
+
+    bool operator==(const parse_error_value_t& other) const;
+    bool operator!=(const parse_error_value_t& other) const;
+};
+
+struct ORCUS_PSR_DLLPUBLIC xml_name_t
+{
+    enum to_string_type { use_alias, use_short_name };
+
+    xmlns_id_t ns;
+    std::string_view name;
+
+    xml_name_t();
+    xml_name_t(xmlns_id_t _ns, std::string_view _name);
+    xml_name_t(const xml_name_t& r);
+
+    xml_name_t& operator= (const xml_name_t& other);
+
+    bool operator== (const xml_name_t& other) const;
+    bool operator!= (const xml_name_t& other) const;
+
+    std::string to_string(const xmlns_context& cxt, to_string_type type) const;
+
+    std::string to_string(const xmlns_repository& repo) const;
 };
 
 struct ORCUS_PSR_DLLPUBLIC xml_token_attr_t
 {
     xmlns_id_t ns;
     xml_token_t name;
-    pstring raw_name;
-    pstring value;
+    std::string_view raw_name;
+    std::string_view value;
 
     /**
      * Whether or not the attribute value is transient. A transient value is
@@ -58,10 +94,10 @@ struct ORCUS_PSR_DLLPUBLIC xml_token_attr_t
 
     xml_token_attr_t();
     xml_token_attr_t(
-        xmlns_id_t _ns, xml_token_t _name, const pstring& _value, bool _transient);
+        xmlns_id_t _ns, xml_token_t _name, std::string_view _value, bool _transient);
     xml_token_attr_t(
-        xmlns_id_t _ns, xml_token_t _name, const pstring& _raw_name,
-        const pstring& _value, bool _transient);
+        xmlns_id_t _ns, xml_token_t _name, std::string_view _raw_name,
+        std::string_view _value, bool _transient);
 };
 
 /**
@@ -72,13 +108,13 @@ struct ORCUS_PSR_DLLPUBLIC xml_token_element_t
 {
     xmlns_id_t ns;
     xml_token_t name;
-    pstring raw_name;
+    std::string_view raw_name;
     std::vector<xml_token_attr_t> attrs;
 
     xml_token_element_t& operator= (xml_token_element_t) = delete;
 
     xml_token_element_t();
-    xml_token_element_t(xmlns_id_t _ns, xml_token_t _name, const pstring& _raw_name, std::vector<xml_token_attr_t>&& _attrs);
+    xml_token_element_t(xmlns_id_t _ns, xml_token_t _name, std::string_view _raw_name, std::vector<xml_token_attr_t>&& _attrs);
     xml_token_element_t(const xml_token_element_t& other);
     xml_token_element_t(xml_token_element_t&& other);
 };
@@ -403,7 +439,8 @@ enum class dump_format_t
     flat,
     html,
     json,
-    xml
+    xml,
+    yaml
 };
 
 struct ORCUS_PSR_DLLPUBLIC length_t
@@ -414,6 +451,9 @@ struct ORCUS_PSR_DLLPUBLIC length_t
     length_t();
 
     std::string to_string() const;
+
+    bool operator== (const length_t& other) const noexcept;
+    bool operator!= (const length_t& other) const noexcept;
 };
 
 struct ORCUS_PSR_DLLPUBLIC date_time_t
@@ -435,17 +475,19 @@ struct ORCUS_PSR_DLLPUBLIC date_time_t
 
     bool operator== (const date_time_t& other) const;
     bool operator!= (const date_time_t& other) const;
+    bool operator< (const date_time_t& other) const;
 
     std::string to_string() const;
 
     void swap(date_time_t& other);
 };
 
-ORCUS_PSR_DLLPUBLIC dump_format_t to_dump_format_enum(const char* p, size_t n);
+ORCUS_PSR_DLLPUBLIC dump_format_t to_dump_format_enum(std::string_view s);
 
-ORCUS_PSR_DLLPUBLIC std::vector<std::pair<pstring, dump_format_t>> get_dump_format_entries();
+ORCUS_PSR_DLLPUBLIC std::vector<std::pair<std::string_view, dump_format_t>> get_dump_format_entries();
 
 ORCUS_PSR_DLLPUBLIC std::ostream& operator<< (std::ostream& os, const date_time_t& v);
+ORCUS_PSR_DLLPUBLIC std::ostream& operator<< (std::ostream& os, format_t v);
 
 typedef ::std::vector<xml_token_attr_t> xml_attrs_t;
 
