@@ -5,10 +5,11 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include "orcus/css_document_tree.hpp"
-#include "orcus/css_types.hpp"
-#include "orcus/stream.hpp"
-#include "orcus/global.hpp"
+#include <orcus/css_document_tree.hpp>
+#include <orcus/css_types.hpp>
+#include <orcus/css_parser_base.hpp>
+#include <orcus/stream.hpp>
+#include <orcus/global.hpp>
 
 #include <cstdlib>
 #include <cassert>
@@ -16,40 +17,107 @@
 #include <sstream>
 #include <iterator>
 
-using namespace orcus;
-using namespace std;
+#include <boost/filesystem.hpp>
 
-bool check_prop(const css_properties_t& props, const pstring& key, const pstring& val)
+using namespace orcus;
+
+namespace fs = boost::filesystem;
+
+bool check_prop(const css_properties_t& props, std::string_view key, std::string_view val)
 {
     css_properties_t::const_iterator it = props.find(key);
     if (it == props.end())
     {
-        cout << "property '" << key << "' not found" << endl;
+        std::cout << "property '" << key << "' not found" << std::endl;
         return false;
     }
 
     // Chain all property values into a single string delimited by a " ".
-    const vector<css_property_value_t>& vals = it->second;
-    ostringstream os;
+    const std::vector<css_property_value_t>& vals = it->second;
+    std::ostringstream os;
     if (vals.size() > 1)
     {
-        vector<css_property_value_t>::const_iterator it_end = vals.end();
-        advance(it_end, -1);
-        copy(vals.begin(), it_end, ostream_iterator<css_property_value_t>(os, " "));
+        auto it_end = vals.end();
+        std::advance(it_end, -1);
+        std::copy(vals.begin(), it_end, std::ostream_iterator<css_property_value_t>(os, " "));
     }
     os << vals.back();
 
-    string val_stored = os.str();
-    if (val.str() != val_stored)
+    std::string val_stored = os.str();
+    if (val != val_stored)
     {
-        cout << "property '" << key << "' is expected to have value '"
-            << val << "' but '" << val_stored << "' is found." << endl;
+        std::cout << "property '" << key << "' is expected to have value '"
+            << val << "' but '" << val_stored << "' is found." << std::endl;
         return false;
     }
 
     return true;
 }
 
+using check_properties_type = std::vector<std::pair<std::string, std::string>>;
+
+bool check_props(const css_properties_t& props, check_properties_type expected)
+{
+    bool pass = true;
+
+    for (const auto& [key, value] : expected)
+    {
+        bool res = check_prop(props, key, value);
+        if (!res)
+            pass = false;
+    }
+
+    return pass;
+}
+
+css_document_tree load_document(const fs::path& path)
+{
+    std::cout << path << std::endl;
+    file_content content(path.string());
+    css_document_tree doc;
+    doc.load(content.str());
+
+    return doc;
+}
+
+void test_css_invalids()
+{
+    // Get all yaml files in this directory.
+    fs::path dirpath(SRCDIR"/test/css/invalids/");
+    fs::directory_iterator it_end;
+
+    size_t file_count = 0;
+
+    for (fs::directory_iterator it(dirpath); it != it_end; ++it)
+    {
+        fs::path path = it->path();
+        if (!fs::is_regular_file(path))
+            continue;
+
+        if (fs::extension(path) != ".css")
+            continue;
+
+        std::cout << "parsing invalid file " << path.filename().string() << "..." << std::endl;
+
+        ++file_count;
+
+        file_content content(path.string().data());
+        css_document_tree doc;
+
+        try
+        {
+            doc.load(content.str());
+            assert(!"css::parse_error was not thrown, but expected to be.");
+        }
+        catch (const css::parse_error&)
+        {
+            // This is expected.
+        }
+    }
+
+    assert(file_count > 0);
+}
+
 void test_css_simple_selector_equality()
 {
     css_simple_selector_t left;
@@ -72,13 +140,19 @@ void test_css_simple_selector_equality()
     assert(left == right);
 }
 
+void test_css_empty()
+{
+    fs::path path = SRCDIR"/test/css/empty.css";
+    std::cout << path << std::endl;
+
+    file_content content(path.string());
+    css_document_tree doc;
+    doc.load(content.str());
+}
+
 void test_css_parse_basic1()
 {
-    const char* path = SRCDIR"/test/css/basic1.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic1.css");
 
     css_selector_t selector;
     selector.first.name = "table";
@@ -104,11 +178,7 @@ void test_css_parse_basic1()
 
 void test_css_parse_basic2()
 {
-    const char* path = SRCDIR"/test/css/basic2.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic2.css");
 
     css_selector_t selector;
     selector.first.name = "div";
@@ -131,11 +201,7 @@ void test_css_parse_basic2()
 
 void test_css_parse_basic3()
 {
-    const char* path = SRCDIR"/test/css/basic3.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic3.css");
 
     css_selector_t selector;
     selector.first.name = "html";
@@ -170,11 +236,7 @@ void test_css_parse_basic3()
 
 void test_css_parse_basic4()
 {
-    const char* path = SRCDIR"/test/css/basic4.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic4.css");
 
     css_selector_t selector;
     selector.first.name = "h1";
@@ -211,11 +273,7 @@ void test_css_parse_basic4()
 
 void test_css_parse_basic5()
 {
-    const char* path = SRCDIR"/test/css/basic5.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic5.css");
 
     css_selector_t selector;
     selector.first.classes.insert("info");
@@ -228,11 +286,7 @@ void test_css_parse_basic5()
 
 void test_css_parse_basic6()
 {
-    const char* path = SRCDIR"/test/css/basic6.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic6.css");
 
     css_selector_t selector;
     selector.first.name = "h1";
@@ -259,11 +313,7 @@ void test_css_parse_basic6()
 
 void test_css_parse_basic7()
 {
-    const char* path = SRCDIR"/test/css/basic7.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic7.css");
 
     css_selector_t selector;
     selector.first.classes.insert("one");
@@ -312,11 +362,7 @@ void test_css_parse_basic7()
 
 void test_css_parse_basic8()
 {
-    const char* path = SRCDIR"/test/css/basic8.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic8.css");
 
     css_selector_t selector;
     selector.first.classes.insert("ribbon");
@@ -374,11 +420,7 @@ void test_css_parse_basic8()
 
 void test_css_parse_basic9()
 {
-    const char* path = SRCDIR"/test/css/basic9.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic9.css");
 
     css_selector_t selector;
     selector.first.name = "a";
@@ -410,11 +452,7 @@ void test_css_parse_basic9()
 
 void test_css_parse_basic10()
 {
-    const char* path = SRCDIR"/test/css/basic10.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic10.css");
 
     css_selector_t selector;
     selector.first.classes.insert("foo");
@@ -428,11 +466,7 @@ void test_css_parse_basic10()
 
 void test_css_parse_basic11()
 {
-    const char* path = SRCDIR"/test/css/basic11.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic11.css");
 
     css_selector_t selector;
     selector.first.classes.insert("callout");
@@ -449,11 +483,7 @@ void test_css_parse_basic11()
 
 void test_css_parse_basic12()
 {
-    const char* path = SRCDIR"/test/css/basic12.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic12.css");
 
     css_selector_t selector;
     selector.first.name = "div";
@@ -488,11 +518,7 @@ void test_css_parse_basic12()
 
 void test_css_parse_basic13()
 {
-    const char* path = SRCDIR"/test/css/basic13.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic13.css");
 
     css_selector_t selector;
     selector.first.id = "p1";
@@ -523,11 +549,7 @@ void test_css_parse_basic13()
 
 void test_css_parse_basic14()
 {
-    const char* path = SRCDIR"/test/css/basic14.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/basic14.css");
 
     css_selector_t selector;
     selector.first.id = "p1";
@@ -558,11 +580,7 @@ void test_css_parse_basic14()
 
 void test_css_parse_chained1()
 {
-    const char* path = SRCDIR"/test/css/chained1.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/chained1.css");
 
     css_selector_t selector;
     selector.first.name = "div";
@@ -593,11 +611,7 @@ void test_css_parse_chained1()
 
 void test_css_parse_chained2()
 {
-    const char* path = SRCDIR"/test/css/chained2.css";
-    cout << path << endl;
-    string strm = load_file_content(path);
-    css_document_tree doc;
-    doc.load(strm);
+    css_document_tree doc = load_document(SRCDIR"/test/css/chained2.css");
 
     // Build selector '#id1 table.data td'.
     css_selector_t selector;
@@ -619,9 +633,42 @@ void test_css_parse_chained2()
     assert(check_prop(*props, "border", "solid 2px"));
 }
 
+void test_css_parse_utf8_1()
+{
+    css_document_tree doc = load_document(SRCDIR"/test/css/utf8-1.css");
+
+    css_document_tree doc2;
+    doc2 = std::move(doc);  // test the move assignment operator.
+
+    css_selector_t selector;
+    selector.first.classes.insert("style17");
+
+    const css_properties_t* props = doc2.get_properties(selector, 0);
+    assert(props);
+    assert(props->size() == 11);
+
+    check_properties_type expected = {
+        { "mso-pattern", "auto none" },
+        { "background", "#EDEDED" },
+        { "mso-style-name", "20% - 强调文字颜色 3" },
+        { "color", "#000000" },
+        { "font-size", "11.0pt" },
+        { "font-weight", "400" },
+        { "font-style", "normal" },
+        { "font-family", "宋体" },
+        { "text-decoration", "none" },
+        { "mso-generic-font-family", "auto" },
+        { "mso-font-charset", "0" },
+    };
+
+    assert(check_props(*props, expected));
+}
+
 int main()
 {
+    test_css_invalids();
     test_css_simple_selector_equality();
+    test_css_empty();
     test_css_parse_basic1();
     test_css_parse_basic2();
     test_css_parse_basic3();
@@ -638,6 +685,7 @@ int main()
     test_css_parse_basic14();
     test_css_parse_chained1();
     test_css_parse_chained2();
+    test_css_parse_utf8_1();
 
     return EXIT_SUCCESS;
 }
