@@ -18,7 +18,7 @@ namespace orcus {
 
 namespace {
 
-class gnumeric_style_region_attr_parser : public std::unary_function<xml_token_attr_t, void>
+class gnumeric_style_region_attr_parser
 {
 public:
     gnumeric_style_region_attr_parser(gnumeric_style_region& style_region_data):
@@ -30,25 +30,25 @@ public:
         {
             case XML_startCol:
             {
-                size_t n = atoi(attr.value.get());
+                size_t n = atoi(attr.value.data());
                 m_style_region_data.start_col = n;
             }
             break;
             case XML_startRow:
             {
-                size_t n = atoi(attr.value.get());
+                size_t n = atoi(attr.value.data());
                 m_style_region_data.start_row = n;
             }
             break;
             case XML_endCol:
             {
-                size_t n = atoi(attr.value.get());
+                size_t n = atoi(attr.value.data());
                 m_style_region_data.end_col = n;
             }
             break;
             case XML_endRow:
             {
-                size_t n = atoi(attr.value.get());
+                size_t n = atoi(attr.value.data());
                 m_style_region_data.end_row = n;
             }
             break;
@@ -61,7 +61,7 @@ private:
     gnumeric_style_region& m_style_region_data;
 };
 
-class gnumeric_font_attr_parser : public std::unary_function<xml_token_attr_t, void>
+class gnumeric_font_attr_parser
 {
 public:
     gnumeric_font_attr_parser(spreadsheet::iface::import_styles& styles) :
@@ -73,25 +73,25 @@ public:
         {
             case XML_Unit:
             {
-                double n = atoi(attr.value.get());
+                double n = atoi(attr.value.data());
                 m_styles.set_font_size(n);
             }
             break;
             case XML_Bold:
             {
-                bool b = atoi(attr.value.get()) != 0;
+                bool b = atoi(attr.value.data()) != 0;
                 m_styles.set_font_bold(b);
             }
             break;
             case XML_Italic:
             {
-                bool b = atoi(attr.value.get()) != 0;
+                bool b = atoi(attr.value.data()) != 0;
                 m_styles.set_font_italic(b);
             }
             break;
             case XML_Underline:
             {
-                int n = atoi(attr.value.get());
+                int n = atoi(attr.value.data());
                 switch (n)
                 {
                     case 0:
@@ -115,7 +115,7 @@ private:
     spreadsheet::iface::import_styles& m_styles;
 };
 
-class gnumeric_style_attr_parser : public std::unary_function<xml_token_attr_t, void>
+class gnumeric_style_attr_parser
 {
 public:
     gnumeric_style_attr_parser(spreadsheet::iface::import_styles& styles, gnumeric_color& front_color) :
@@ -132,7 +132,7 @@ public:
             {
                 spreadsheet::color_elem_t red, green, blue;
                 gnumeric_helper::parse_RGB_color_attribute(red, green, blue, attr.value);
-                m_styles.set_fill_fg_color(0, red, green, blue);
+                m_styles.set_fill_fg_color(255, red, green, blue);
 
                 m_fill = true;
 
@@ -145,14 +145,14 @@ public:
             {
                 spreadsheet::color_elem_t red, green, blue;
                 gnumeric_helper::parse_RGB_color_attribute(red, green, blue, attr.value);
-                m_styles.set_fill_bg_color(0, red, green, blue);
+                m_styles.set_fill_bg_color(255, red, green, blue);
 
                 m_fill = true;
             }
             break;
             case XML_Hidden:
             {
-                bool b = atoi(attr.value.get());
+                bool b = atoi(attr.value.data());
                 m_styles.set_cell_hidden(b);
 
                 m_protection = true;
@@ -160,7 +160,7 @@ public:
             break;
             case XML_Locked:
             {
-                bool b = atoi(attr.value.get());
+                bool b = atoi(attr.value.data());
                 m_styles.set_cell_locked(b);
 
                 m_protection = true;
@@ -170,7 +170,7 @@ public:
             {
                 if (attr.value != "General")
                 {
-                    m_styles.set_number_format_code(attr.value.get(), attr.value.size());
+                    m_styles.set_number_format_code(attr.value);
                     size_t index = m_styles.commit_number_format();
                     m_styles.set_xf_number_format(index);
                 }
@@ -290,7 +290,7 @@ spreadsheet::condition_operator_t get_condition_operator(int val)
     return orcus::spreadsheet::condition_operator_t::unknown;
 }
 
-class gnumeric_condition_attr_parser : public std::unary_function<xml_token_attr_t, void>
+class gnumeric_condition_attr_parser
 {
 public:
     gnumeric_condition_attr_parser(spreadsheet::iface::import_conditional_format* cond_format):
@@ -302,7 +302,7 @@ public:
         {
             case XML_Operator:
             {
-                int val = atoi(attr.value.get());
+                int val = atoi(attr.value.data());
                 spreadsheet::condition_operator_t op = get_condition_operator(val);
                 m_cond_format->set_operator(op);
             }
@@ -316,7 +316,7 @@ private:
     spreadsheet::iface::import_conditional_format* m_cond_format;
 };
 
-class gnumeric_col_row_info : public std::unary_function<xml_token_attr_t, void>
+class gnumeric_col_row_info
 {
 public:
     gnumeric_col_row_info() :
@@ -331,25 +331,25 @@ public:
         {
             case XML_No:
             {
-                size_t i = atoi(attr.value.get());
+                size_t i = atoi(attr.value.data());
                 m_position = i;
             }
             break;
             case XML_Unit:
             {
-                double n = atof(attr.value.get());
+                double n = atof(attr.value.data());
                 m_size = n;
             }
             break;
             case XML_Count:
             {
-                size_t i = atoi(attr.value.get());
+                size_t i = atoi(attr.value.data());
                 m_num_repeated = i;
             }
             break;
             case XML_Hidden:
             {
-                bool b = atoi(attr.value.get()) != 0;
+                bool b = atoi(attr.value.data()) != 0;
                 m_hidden = b;
             }
         }
@@ -383,28 +383,6 @@ private:
 
 };
 
-class gnumeric_autofilter_attr_parser : public std::unary_function<xml_token_attr_t, void>
-{
-public:
-    gnumeric_autofilter_attr_parser(spreadsheet::iface::import_auto_filter& auto_filter):
-        m_auto_filter(auto_filter) {}
-
-    void operator() (const xml_token_attr_t& attr)
-    {
-        switch(attr.name)
-        {
-            case XML_Area:
-                m_auto_filter.set_range(attr.value.get(), attr.value.size());
-            break;
-            default:
-                ;
-        }
-    }
-
-private:
-    spreadsheet::iface::import_auto_filter& m_auto_filter;
-};
-
 enum gnumeric_filter_field_op_t
 {
     filter_equal,
@@ -424,7 +402,7 @@ enum gnumeric_filter_field_type_t
     filter_type_invalid
 };
 
-class gnumeric_autofilter_field_attr_parser : public std::unary_function<xml_token_attr_t, void>
+class gnumeric_autofilter_field_attr_parser
 {
 public:
     gnumeric_autofilter_field_attr_parser(spreadsheet::iface::import_auto_filter& auto_filter):
@@ -438,7 +416,7 @@ public:
         {
             case XML_Index:
             {
-                spreadsheet::col_t col = atoi(attr.value.get());                
+                spreadsheet::col_t col = atoi(attr.value.data());
                 m_auto_filter.set_column(col);
             }
             break;
@@ -510,7 +488,7 @@ private:
                 m_filter_value_type == "40" ||
                 m_filter_value_type == "60" )
         {
-            m_auto_filter.append_column_match_value(m_filter_value.get(), m_filter_value.size());
+            m_auto_filter.append_column_match_value(m_filter_value);
         }
     }
 
@@ -540,14 +518,6 @@ gnumeric_sheet_context::~gnumeric_sheet_context()
 {
 }
 
-bool gnumeric_sheet_context::can_handle_element(xmlns_id_t ns, xml_token_t name) const
-{
-    if (ns == NS_gnumeric_gnm && name == XML_Cells)
-        return false;
-
-    return true;
-}
-
 xml_context_base* gnumeric_sheet_context::create_child_context(xmlns_id_t ns, xml_token_t name)
 {
     if (ns == NS_gnumeric_gnm && name == XML_Cells)
@@ -591,12 +561,28 @@ void gnumeric_sheet_context::start_element(xmlns_id_t ns, xml_token_t name, cons
             break;
             case XML_Filter:
             {
+                spreadsheet::iface::import_reference_resolver* resolver =
+                    mp_factory->get_reference_resolver(spreadsheet::formula_ref_context_t::global);
+
                 mp_auto_filter = mp_sheet->get_auto_filter();
-                if (mp_auto_filter)
+
+                if (resolver && mp_auto_filter)
                 {
-                    std::for_each(attrs.begin(), attrs.end(),
-                            gnumeric_autofilter_attr_parser(
-                                *mp_auto_filter));
+                    for (const xml_token_attr_t& attr : attrs)
+                    {
+                        switch (attr.name)
+                        {
+                            case XML_Area:
+                            {
+                                spreadsheet::range_t range = to_rc_range(
+                                    resolver->resolve_range(attr.value));
+                                mp_auto_filter->set_range(range);
+                            }
+                            break;
+                            default:
+                                ;
+                        }
+                    }
                 }
             }
             break;
@@ -695,7 +681,7 @@ bool gnumeric_sheet_context::end_element(xmlns_id_t ns, xml_token_t name)
     return pop_stack(ns, name);
 }
 
-void gnumeric_sheet_context::characters(const pstring& str, bool transient)
+void gnumeric_sheet_context::characters(std::string_view str, bool transient)
 {
     if (transient)
         chars = m_pool.intern(str).first;
@@ -772,14 +758,14 @@ void gnumeric_sheet_context::start_condition(const xml_attrs_t& attrs)
 
 void gnumeric_sheet_context::end_table()
 {
-    mp_sheet = mp_factory->append_sheet(m_sheet_index, chars.get(), chars.size());
+    mp_sheet = mp_factory->append_sheet(m_sheet_index, chars);
 }
 
 void gnumeric_sheet_context::end_font()
 {
     spreadsheet::iface::import_styles& styles = *mp_factory->get_styles();
     styles.set_font_color(0, front_color.red, front_color.green, front_color.blue);
-    styles.set_font_name(chars.get(), chars.size());
+    styles.set_font_name(chars);
     size_t font_id = styles.commit_font();
     styles.set_xf_font(font_id);
 }
@@ -838,7 +824,7 @@ void gnumeric_sheet_context::end_expression()
         mp_sheet->get_conditional_format();
     if (cond_format)
     {
-        cond_format->set_formula(chars.get(), chars.size());
+        cond_format->set_formula(chars);
         cond_format->commit_condition();
     }
 }
