@@ -18,31 +18,30 @@
 #include "session_context.hpp"
 
 #include "xml_stream_parser.hpp"
-#include "xml_simple_stream_handler.hpp"
 
 namespace orcus {
 
-void import_ods::read_styles(const char* p, size_t n, spreadsheet::iface::import_styles* styles)
+void import_ods::read_styles(std::string_view s, spreadsheet::iface::import_styles* styles)
 {
     if(!styles)
         return;
 
-    if (!p || !n)
+    if (s.empty())
         return;
 
     session_context cxt;
     odf_styles_map_type styles_map;
-    auto context = orcus::make_unique<styles_context>(cxt, odf_tokens, styles_map, styles);
+    auto context = std::make_unique<styles_context>(cxt, odf_tokens, styles_map, styles);
 
-    xml_simple_stream_handler stream_handler(context.release());
+    xml_stream_handler stream_handler(cxt, odf_tokens, std::move(context));
 
     xmlns_repository ns_repo;
     ns_repo.add_predefined_values(NS_odf_all);
 
-    orcus::config config;
+    orcus::config config(format_t::ods);
     xml_stream_parser parser(
         config, ns_repo, odf_tokens,
-        p, n);
+        s.data(), s.size());
     parser.set_handler(&stream_handler);
     parser.parse();
 }
