@@ -5,16 +5,16 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include "orcus/orcus_xml.hpp"
-#include "orcus/global.hpp"
-#include "orcus/sax_ns_parser.hpp"
-#include "orcus/spreadsheet/import_interface.hpp"
-#include "orcus/spreadsheet/export_interface.hpp"
-#include "orcus/xml_namespace.hpp"
-#include "orcus/stream.hpp"
-#include "orcus/string_pool.hpp"
+#include <orcus/orcus_xml.hpp>
+#include <orcus/global.hpp>
+#include <orcus/sax_ns_parser.hpp>
+#include <orcus/spreadsheet/import_interface.hpp>
+#include <orcus/spreadsheet/export_interface.hpp>
+#include <orcus/stream.hpp>
+#include <orcus/string_pool.hpp>
+#include "pstring.hpp"
 
-#include "xml_map_tree.hpp"
+#include "orcus_xml_impl.hpp"
 
 #define ORCUS_DEBUG_XML 0
 
@@ -35,21 +35,21 @@ class xml_data_sax_handler
 {
     struct scope
     {
-        xmlns_id_t ns;
-        pstring name;
+        xml_name_t name;
         std::ptrdiff_t element_open_begin;
         std::ptrdiff_t element_open_end;
 
         xml_map_tree::element_type type;
 
-        scope(xmlns_id_t _ns, const pstring& _name) :
-            ns(_ns), name(_name),
-            element_open_begin(0), element_open_end(0),
+        scope(xmlns_id_t _ns, std::string_view _name) :
+            name(_ns, _name),
+            element_open_begin(0),
+            element_open_end(0),
             type(xml_map_tree::element_unknown) {}
     };
 
-    vector<sax_ns_parser_attribute> m_attrs;
-    vector<scope> m_scopes;
+    std::vector<sax_ns_parser_attribute> m_attrs;
+    std::vector<scope> m_scopes;
 
     string_pool m_pool;
     spreadsheet::iface::import_factory& m_factory;
@@ -57,91 +57,101 @@ class xml_data_sax_handler
     const xml_map_tree& m_map_tree;
     xml_map_tree::walker m_map_tree_walker;
 
-    const xml_map_tree::element* mp_current_elem;
-    pstring m_current_chars;
-    bool m_in_range_ref:1;
+    xml_map_tree::element* mp_current_elem;
+    std::string_view m_current_chars;
+    bool m_in_range_ref;
+    xml_map_tree::range_reference* mp_increment_row;
 
 private:
 
-    const sax_ns_parser_attribute* find_attr_by_name(xmlns_id_t ns, const pstring& name)
+    const sax_ns_parser_attribute* find_attr_by_name(const xml_name_t& name)
     {
         for (const sax_ns_parser_attribute& attr : m_attrs)
         {
-            if (attr.ns == ns && attr.name == name)
+            if (attr.ns == name.ns && attr.name == name.name)
                 return &attr;
         }
         return nullptr;
     }
 
-    void set_single_link_cell(const xml_map_tree::cell_reference& ref, const pstring& val)
+    void set_single_link_cell(const xml_map_tree::cell_reference& ref, std::string_view val)
     {
-        spreadsheet::iface::import_sheet* sheet = m_factory.get_sheet(ref.pos.sheet.get(), ref.pos.sheet.size());
+        spreadsheet::iface::import_sheet* sheet = m_factory.get_sheet(ref.pos.sheet);
         if (sheet)
-            sheet->set_auto(ref.pos.row, ref.pos.col, val.get(), val.size());
+            sheet->set_auto(ref.pos.row, ref.pos.col, val);
     }
 
-    void set_field_link_cell(const xml_map_tree::field_in_range& field, const pstring& val)
+    void set_field_link_cell(xml_map_tree::field_in_range& field, std::string_view val)
     {
         assert(field.ref);
         assert(!field.ref->pos.sheet.empty());
 
         const xml_map_tree::cell_position& pos = field.ref->pos;
-        spreadsheet::iface::import_sheet* sheet = m_factory.get_sheet(pos.sheet.get(), pos.sheet.size());
+        spreadsheet::iface::import_sheet* sheet = m_factory.get_sheet(pos.sheet);
         if (sheet)
             sheet->set_auto(
-               pos.row + field.ref->row_size,
+               pos.row + field.ref->row_position,
                pos.col + field.column_pos,
-               val.get(), val.size());
+               val);
     }
 
 public:
     xml_data_sax_handler(
-       spreadsheet::iface::import_factory& factory,
-       xml_map_tree::const_element_list_type& link_positions,
-       const xml_map_tree& map_tree) :
+        spreadsheet::iface::import_factory& factory,
+        xml_map_tree::const_element_list_type& link_positions,
+        const xml_map_tree& map_tree) :
         m_factory(factory),
         m_link_positions(link_positions),
         m_map_tree(map_tree),
         m_map_tree_walker(map_tree.get_tree_walker()),
         mp_current_elem(nullptr),
-        m_in_range_ref(false) {}
+        m_in_range_ref(false),
+        mp_increment_row(nullptr) {}
 
     void doctype(const sax::doctype_declaration&)
     {
     }
 
-    void start_declaration(const pstring&)
+    void start_declaration(std::string_view)
     {
     }
 
-    void end_declaration(const pstring&)
+    void end_declaration(std::string_view)
     {
         m_attrs.clear();
     }
 
     void start_element(const sax_ns_parser_element& elem)
     {
-        m_scopes.push_back(scope(elem.ns, elem.name));
+        m_scopes.emplace_back(elem.ns, elem.name);
         scope& cur = m_scopes.back();
         cur.element_open_begin = elem.begin_pos;
         cur.element_open_end = elem.end_pos;
-        m_current_chars.clear();
+        m_current_chars = std::string_view{};
 
-        mp_current_elem = m_map_tree_walker.push_element(elem.ns, elem.name);
+        mp_current_elem = m_map_tree_walker.push_element({elem.ns, elem.name});
         if (mp_current_elem)
         {
+            if (mp_current_elem->row_group && mp_increment_row == mp_current_elem->row_group)
+            {
+                // The last closing element was a row group boundary.  Increment the row position.
+                xml_map_tree::range_reference* ref = mp_current_elem->row_group;
+                ++ref->row_position;
+                mp_increment_row = nullptr;
+            }
+
             // Go through all linked attributes that belong to this element,
             // and see if they exist in this content xml.
             for (const auto& p_attr : mp_current_elem->attributes)
             {
                 const xml_map_tree::attribute& linked_attr = *p_attr;
-                const sax_ns_parser_attribute* p = find_attr_by_name(linked_attr.ns, linked_attr.name);
+                const sax_ns_parser_attribute* p = find_attr_by_name(linked_attr.name);
                 if (!p)
                     continue;
 
                 // This attribute is linked. Import its value.
 
-                pstring val_trimmed = p->value.trim();
+                std::string_view val_trimmed = trim(p->value);
                 switch (linked_attr.ref_type)
                 {
                     case xml_map_tree::reference_cell:
@@ -149,9 +159,6 @@ public:
                         break;
                     case xml_map_tree::reference_range_field:
                     {
-                        const xml_map_tree::field_in_range& field = *linked_attr.field_ref;
-                        if (field.column_pos == 0)
-                            ++field.ref->row_size;
                         set_field_link_cell(*linked_attr.field_ref, val_trimmed);
                         break;
                     }
@@ -184,9 +191,6 @@ public:
                 }
                 case xml_map_tree::reference_range_field:
                 {
-                    const xml_map_tree::field_in_range& field = *mp_current_elem->field_ref;
-                    if (field.column_pos == 0)
-                        ++field.ref->row_size;
                     set_field_link_cell(*mp_current_elem->field_ref, m_current_chars);
                     break;
                 }
@@ -194,6 +198,36 @@ public:
                     ;
             }
 
+            if (mp_current_elem->row_group)
+            {
+                // This element defines a row-group boundary.
+                spreadsheet::row_t row_start = mp_current_elem->row_group_position;
+                spreadsheet::row_t row_end = mp_current_elem->row_group->row_position - 1;
+                if (row_end > row_start)
+                {
+                    // This is the end of a parent row-group.  Fill down the
+                    // cell values.
+                    const xml_map_tree::range_reference& ref = *mp_current_elem->row_group;
+
+                    spreadsheet::iface::import_sheet* sheet = m_factory.get_sheet(ref.pos.sheet);
+
+                    if (sheet)
+                    {
+                        row_start += ref.pos.row + 1;
+                        row_end += ref.pos.row + 1;
+
+                        for (spreadsheet::col_t col : mp_current_elem->linked_range_fields)
+                        {
+                            col += ref.pos.col;
+                            sheet->fill_down_cells(row_start, col, row_end - row_start);
+                        }
+                    }
+                }
+
+                mp_current_elem->row_group_position = mp_current_elem->row_group->row_position;
+                mp_increment_row = mp_current_elem->row_group;
+            }
+
             // Store the end element position in stream for linked elements.
             const scope& cur = m_scopes.back();
             if (mp_current_elem->ref_type == xml_map_tree::reference_cell ||
@@ -217,20 +251,20 @@ public:
         }
 
         m_scopes.pop_back();
-        mp_current_elem = m_map_tree_walker.pop_element(elem.ns, elem.name);
+        mp_current_elem = m_map_tree_walker.pop_element({elem.ns, elem.name});
     }
 
-    void characters(const pstring& val, bool transient)
+    void characters(std::string_view val, bool transient)
     {
         if (!mp_current_elem)
             return;
 
-        m_current_chars = val.trim();
+        m_current_chars = trim(val);
         if (transient)
             m_current_chars = m_pool.intern(m_current_chars).first;
     }
 
-    void attribute(const pstring& /*name*/, const pstring& /*val*/)
+    void attribute(std::string_view /*name*/, std::string_view /*val*/)
     {
         // Ignore attributes in XML declaration.
     }
@@ -257,11 +291,13 @@ struct scope
     scope(const xml_map_tree::element& _elem) :
         element(_elem), opened(false)
     {
-        end_child_pos = element.child_elements->end();
         current_child_pos = end_child_pos;
 
         if (element.elem_type == xml_map_tree::element_unlinked)
+        {
             current_child_pos = element.child_elements->begin();
+            end_child_pos = element.child_elements->end();
+        }
     }
 };
 
@@ -314,8 +350,7 @@ void write_opening_element(
 
         const xml_map_tree::cell_position& pos = attr.cell_ref->pos;
 
-        const spreadsheet::iface::export_sheet* sheet =
-            fact.get_sheet(pos.sheet.get(), pos.sheet.size());
+        const spreadsheet::iface::export_sheet* sheet = fact.get_sheet(pos.sheet);
         if (!sheet)
             continue;
 
@@ -343,14 +378,14 @@ void write_range_reference_group(
    ostream& os, const xml_map_tree::element& root, const xml_map_tree::range_reference& ref,
    const spreadsheet::iface::export_factory& factory)
 {
-    const spreadsheet::iface::export_sheet* sheet = factory.get_sheet(ref.pos.sheet.get(), ref.pos.sheet.size());
+    const spreadsheet::iface::export_sheet* sheet = factory.get_sheet(ref.pos.sheet);
     if (!sheet)
         return;
 
     scopes_type scopes;
-    for (spreadsheet::row_t current_row = 0; current_row < ref.row_size; ++current_row)
+    for (spreadsheet::row_t current_row = 0; current_row < ref.row_position; ++current_row)
     {
-        scopes.push_back(orcus::make_unique<scope>(root)); // root element
+        scopes.push_back(std::make_unique<scope>(root)); // root element
 
         while (!scopes.empty())
         {
@@ -385,7 +420,7 @@ void write_range_reference_group(
                     // This is a non-leaf element.  Push a new scope with this
                     // element and re-start the loop.
                     ++cur_scope.current_child_pos;
-                    scopes.push_back(orcus::make_unique<scope>(child_elem));
+                    scopes.push_back(std::make_unique<scope>(child_elem));
                     new_scope = true;
                     break;
                 }
@@ -440,7 +475,7 @@ void write_range_reference(ostream& os, const xml_map_tree::element& elem_top, c
        os, **elem_top.child_elements->begin(), *elem_top.range_parent, factory);
 }
 
-struct less_by_opening_elem_pos : std::binary_function<xml_map_tree::element*, xml_map_tree::element*, bool>
+struct less_by_opening_elem_pos
 {
     bool operator() (const xml_map_tree::element* left, const xml_map_tree::element* right) const
     {
@@ -448,143 +483,103 @@ struct less_by_opening_elem_pos : std::binary_function<xml_map_tree::element*, x
     }
 };
 
-}
-
-struct orcus_xml_impl
-{
-    spreadsheet::iface::import_factory* mp_import_factory;
-    spreadsheet::iface::export_factory* mp_export_factory;
-
-    /** xml namespace repository for the whole session. */
-    xmlns_repository& m_ns_repo;
-
-    /** xml namespace context  */
-    xmlns_context m_ns_cxt_map;
-
-    /** xml element tree that represents all mapped paths. */
-    xml_map_tree m_map_tree;
-
-    spreadsheet::sheet_t m_sheet_count;
-
-    /**
-     * Positions of all linked elements, single and range reference alike.
-     * Stored link elements must be sorted in order of stream positions, and
-     * as such, no linked elements should be nested; there should never be a
-     * linked element inside the substructure of another linked element.
-     */
-    xml_map_tree::const_element_list_type m_link_positions;
-
-    xml_map_tree::cell_position m_cur_range_ref;
-
-    explicit orcus_xml_impl(xmlns_repository& ns_repo) :
-        mp_import_factory(nullptr),
-        mp_export_factory(nullptr),
-        m_ns_repo(ns_repo),
-        m_ns_cxt_map(ns_repo.create_context()),
-        m_map_tree(m_ns_repo),
-        m_sheet_count(0) {}
-};
+} // anonymous namespace
 
 orcus_xml::orcus_xml(xmlns_repository& ns_repo, spreadsheet::iface::import_factory* im_fact, spreadsheet::iface::export_factory* ex_fact) :
-    mp_impl(new orcus_xml_impl(ns_repo))
+    mp_impl(std::make_unique<impl>(ns_repo))
 {
-    mp_impl->mp_import_factory = im_fact;
-    mp_impl->mp_export_factory = ex_fact;
+    mp_impl->im_factory = im_fact;
+    mp_impl->ex_factory = ex_fact;
 }
 
-orcus_xml::~orcus_xml()
+orcus_xml::~orcus_xml() {}
+
+void orcus_xml::set_namespace_alias(std::string_view alias, std::string_view uri, bool default_ns)
 {
-    delete mp_impl;
+    mp_impl->map_tree.set_namespace_alias(alias, uri, default_ns);
 }
 
-void orcus_xml::set_namespace_alias(const pstring& alias, const pstring& uri)
+void orcus_xml::set_cell_link(std::string_view xpath, std::string_view sheet, spreadsheet::row_t row, spreadsheet::col_t col)
 {
-    mp_impl->m_map_tree.set_namespace_alias(alias, uri);
+    pstring sheet_safe = mp_impl->map_tree.intern_string(sheet);
+    mp_impl->map_tree.set_cell_link(xpath, xml_map_tree::cell_position(sheet_safe, row, col));
 }
 
-void orcus_xml::set_cell_link(const pstring& xpath, const pstring& sheet, spreadsheet::row_t row, spreadsheet::col_t col)
+void orcus_xml::start_range(std::string_view sheet, spreadsheet::row_t row, spreadsheet::col_t col)
 {
-    pstring sheet_safe = mp_impl->m_map_tree.intern_string(sheet);
-    mp_impl->m_map_tree.set_cell_link(xpath, xml_map_tree::cell_position(sheet_safe, row, col));
+    pstring sheet_safe = mp_impl->map_tree.intern_string(sheet);
+    mp_impl->cur_range_ref = xml_map_tree::cell_position(sheet_safe, row, col);
+    mp_impl->map_tree.start_range(mp_impl->cur_range_ref);
 }
 
-void orcus_xml::start_range(const pstring& sheet, spreadsheet::row_t row, spreadsheet::col_t col)
+void orcus_xml::append_field_link(std::string_view xpath, std::string_view label)
 {
-    pstring sheet_safe = mp_impl->m_map_tree.intern_string(sheet);
-    mp_impl->m_cur_range_ref = xml_map_tree::cell_position(sheet_safe, row, col);
-    mp_impl->m_map_tree.start_range();
+    mp_impl->map_tree.append_range_field_link(xpath, label);
 }
 
-void orcus_xml::append_field_link(const pstring& xpath)
+void orcus_xml::set_range_row_group(std::string_view xpath)
 {
-    mp_impl->m_map_tree.append_range_field_link(xpath, mp_impl->m_cur_range_ref);
+    mp_impl->map_tree.set_range_row_group(xpath);
 }
 
 void orcus_xml::commit_range()
 {
-    mp_impl->m_cur_range_ref = xml_map_tree::cell_position();
-    mp_impl->m_map_tree.commit_range();
+    mp_impl->cur_range_ref = xml_map_tree::cell_position();
+    mp_impl->map_tree.commit_range();
 }
 
-void orcus_xml::append_sheet(const pstring& name)
+void orcus_xml::append_sheet(std::string_view name)
 {
     if (name.empty())
         return;
 
-    mp_impl->mp_import_factory->append_sheet(mp_impl->m_sheet_count++, name.get(), name.size());
+    mp_impl->im_factory->append_sheet(mp_impl->sheet_count++, name);
 }
 
-void orcus_xml::read_stream(const char* p, size_t n)
+void orcus_xml::read_stream(std::string_view stream)
 {
-#if ORCUS_DEBUG_XML
-    cout << "reading file " << filepath << endl;
-#endif
-
-    pstring strm(p, n);
-    read_impl(strm);
-}
-
-void orcus_xml::read_impl(const pstring& strm)
-{
-    if (strm.empty())
+    if (stream.empty())
         return;
 
     // Insert the range headers and reset the row size counters.
-    xml_map_tree::range_ref_map_type& range_refs = mp_impl->m_map_tree.get_range_references();
+    xml_map_tree::range_ref_map_type& range_refs = mp_impl->map_tree.get_range_references();
+
     for (const auto& ref_pair : range_refs)
     {
         const xml_map_tree::cell_position& ref = ref_pair.first;
         xml_map_tree::range_reference& range_ref = *ref_pair.second;
-        range_ref.row_size = 0; // Reset the row offset.
+        range_ref.row_position = 1; // Reset the row offset.
 
-        spreadsheet::iface::import_sheet* sheet =
-            mp_impl->mp_import_factory->get_sheet(ref.sheet.get(), ref.sheet.size());
+        spreadsheet::iface::import_sheet* sheet = mp_impl->im_factory->get_sheet(ref.sheet);
 
         if (!sheet)
             continue;
 
-        xml_map_tree::const_linkable_list_type::const_iterator it = range_ref.field_nodes.begin(), it_end = range_ref.field_nodes.end();
         spreadsheet::row_t row = ref.row;
         spreadsheet::col_t col = ref.col;
-        for (; it != it_end; ++it)
+
+        for (const xml_map_tree::linkable* e : range_ref.field_nodes)
         {
-            const xml_map_tree::linkable& e = **it;
-            ostringstream os;
-            if (e.ns)
-                os << mp_impl->m_ns_repo.get_short_name(e.ns) << ':';
-            os << e.name;
-            string s = os.str();
-            if (!s.empty())
-                sheet->set_auto(row, col++, &s[0], s.size());
+            if (e->label.empty())
+            {
+                // No custom header label. Create a label from the name of the linkable.
+                std::string s = e->name.to_string(mp_impl->ns_repo);
+                if (!s.empty())
+                    sheet->set_auto(row, col, s);
+            }
+            else
+                sheet->set_auto(row, col, e->label);
+
+            ++col;
         }
     }
 
     // Parse the content xml.
-    xmlns_context ns_cxt = mp_impl->m_ns_repo.create_context(); // new ns context for the content xml stream.
+    xmlns_context ns_cxt = mp_impl->ns_repo.create_context(); // new ns context for the content xml stream.
     xml_data_sax_handler handler(
-       *mp_impl->mp_import_factory, mp_impl->m_link_positions, mp_impl->m_map_tree);
+       *mp_impl->im_factory, mp_impl->link_positions, mp_impl->map_tree);
 
-    sax_ns_parser<xml_data_sax_handler> parser(strm.data(), strm.size(), ns_cxt, handler);
+    sax_ns_parser<xml_data_sax_handler> parser(stream.data(), stream.size(), ns_cxt, handler);
     parser.parse();
 }
 
@@ -601,17 +596,17 @@ void dump_links(const xml_map_tree::const_element_list_type& links)
 
 #endif
 
-void orcus_xml::write(const char* p_in, size_t n_in, std::ostream& out) const
+void orcus_xml::write(std::string_view stream, std::ostream& out) const
 {
-    if (!mp_impl->mp_export_factory)
+    if (!mp_impl->ex_factory)
         // We can't export data witout export factory.
         return;
 
-    if (!n_in)
+    if (stream.empty())
         // Source input stream is empty.
         return;
 
-    xml_map_tree::const_element_list_type& links = mp_impl->m_link_positions;
+    xml_map_tree::const_element_list_type& links = mp_impl->link_positions;
     if (links.empty())
         // nothing to write.
         return;
@@ -619,16 +614,14 @@ void orcus_xml::write(const char* p_in, size_t n_in, std::ostream& out) const
     // Sort all link position by opening element positions.
     std::sort(links.begin(), links.end(), less_by_opening_elem_pos());
 
-    spreadsheet::iface::export_factory& fact = *mp_impl->mp_export_factory;
+    spreadsheet::iface::export_factory& fact = *mp_impl->ex_factory;
     xml_map_tree::const_element_list_type::const_iterator it = links.begin(), it_end = links.end();
 
 #if ORCUS_DEBUG_XML
     dump_links(links);
 #endif
 
-    pstring strm(p_in, n_in);
-
-    const char* p0 = p_in;
+    const char* p0 = stream.data();
     std::ptrdiff_t begin_pos = 0;
 
     for (; it != it_end; ++it)
@@ -639,8 +632,7 @@ void orcus_xml::write(const char* p_in, size_t n_in, std::ostream& out) const
             // Single cell link
             const xml_map_tree::cell_position& pos = elem.cell_ref->pos;
 
-            const spreadsheet::iface::export_sheet* sheet =
-                fact.get_sheet(pos.sheet.get(), pos.sheet.size());
+            const spreadsheet::iface::export_sheet* sheet = fact.get_sheet(pos.sheet);
             if (!sheet)
                 continue;
 
@@ -653,7 +645,7 @@ void orcus_xml::write(const char* p_in, size_t n_in, std::ostream& out) const
 
             write_opening_element(out, elem, fact, false);
             sheet->write_string(out, pos.row, pos.col);
-            out << pstring(p0+close_begin, close_end-close_begin); // closing element.
+            out << std::string_view(p0+close_begin, close_end-close_begin); // closing element.
             begin_pos = close_end;
         }
         else if (elem.range_parent)
@@ -662,8 +654,7 @@ void orcus_xml::write(const char* p_in, size_t n_in, std::ostream& out) const
             const xml_map_tree::range_reference& ref = *elem.range_parent;
             const xml_map_tree::cell_position& pos = ref.pos;
 
-            const spreadsheet::iface::export_sheet* sheet =
-                fact.get_sheet(pos.sheet.get(), pos.sheet.size());
+            const spreadsheet::iface::export_sheet* sheet = fact.get_sheet(pos.sheet);
             if (!sheet)
                 continue;
 
@@ -672,11 +663,11 @@ void orcus_xml::write(const char* p_in, size_t n_in, std::ostream& out) const
             std::ptrdiff_t close_end   = elem.stream_pos.close_end;
 
             assert(open_begin > begin_pos);
-            out << pstring(p0+begin_pos, open_begin-begin_pos); // stream since last linked element.
+            out << std::string_view(p0+begin_pos, open_begin-begin_pos); // stream since last linked element.
 
             write_opening_element(out, elem, fact, false);
             write_range_reference(out, elem, fact);
-            out << pstring(p0+close_begin, close_end-close_begin); // closing element.
+            out << std::string_view(p0+close_begin, close_end-close_begin); // closing element.
             begin_pos = close_end;
         }
         else if (elem.unlinked_attribute_anchor())
@@ -690,7 +681,7 @@ void orcus_xml::write(const char* p_in, size_t n_in, std::ostream& out) const
             bool self_close = elem.stream_pos.open_begin == elem.stream_pos.close_begin;
 
             assert(open_begin > begin_pos);
-            out << pstring(p0+begin_pos, open_begin-begin_pos); // stream since last linked element.
+            out << std::string_view(p0+begin_pos, open_begin-begin_pos); // stream since last linked element.
 
             write_opening_element(out, elem, fact, self_close);
             begin_pos = open_end;
@@ -700,7 +691,7 @@ void orcus_xml::write(const char* p_in, size_t n_in, std::ostream& out) const
     }
 
     // Flush the remaining stream.
-    out << pstring(p0+begin_pos, strm.size()-begin_pos);
+    out << std::string_view(p0+begin_pos, stream.size()-begin_pos);
 }
 
 }
