@@ -19,7 +19,7 @@ session_context::~session_context()
     mp_data.reset();
 }
 
-pstring session_context::intern(const xml_token_attr_t& attr)
+std::string_view session_context::intern(const xml_token_attr_t& attr)
 {
     if (!attr.transient)
         return attr.value;
@@ -27,7 +27,7 @@ pstring session_context::intern(const xml_token_attr_t& attr)
     return m_string_pool.intern(attr.value).first;
 }
 
-pstring session_context::intern(const pstring& s)
+std::string_view session_context::intern(std::string_view s)
 {
     return m_string_pool.intern(s).first;
 }
