@@ -65,7 +65,7 @@ private:
     size_t m_unique_count;
 };
 
-class color_attr_parser : unary_function<xml_token_attr_t, void>
+class color_attr_parser
 {
     pstring m_rgb;
 public:
@@ -94,11 +94,6 @@ xlsx_shared_strings_context::xlsx_shared_strings_context(session_context& sessio
 
 xlsx_shared_strings_context::~xlsx_shared_strings_context() {}
 
-bool xlsx_shared_strings_context::can_handle_element(xmlns_id_t /*ns*/, xml_token_t /*name*/) const
-{
-    return true;
-}
-
 xml_context_base* xlsx_shared_strings_context::create_child_context(xmlns_id_t /*ns*/, xml_token_t /*name*/)
 {
     return nullptr;
@@ -177,8 +172,8 @@ void xlsx_shared_strings_context::start_element(xmlns_id_t ns, xml_token_t name,
         {
             // font
             xml_element_expected(parent, NS_ooxml_xlsx, XML_rPr);
-            pstring font = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_val)).get_value();
-            mp_strings->set_segment_font_name(font.get(), font.size());
+            std::string_view font = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_val)).get_value();
+            mp_strings->set_segment_font_name(font);
         }
         break;
         case XML_family:
@@ -192,10 +187,12 @@ void xlsx_shared_strings_context::start_element(xmlns_id_t ns, xml_token_t name,
         case XML_t:
         {
             // actual text stored as its content.
-            xml_elem_stack_t allowed;
-            allowed.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_si));
-            allowed.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_r));
-            xml_element_expected(parent, allowed);
+            const xml_elem_set_t expected = {
+                { NS_ooxml_xlsx, XML_r },
+                { NS_ooxml_xlsx, XML_rPh },
+                { NS_ooxml_xlsx, XML_si },
+            };
+            xml_element_expected(parent, expected);
         }
         break;
         default:
@@ -216,7 +213,7 @@ bool xlsx_shared_strings_context::end_element(xmlns_id_t ns, xml_token_t name)
             mp_strings->set_segment_italic(true);
         break;
         case XML_r:
-            mp_strings->append_segment(m_cur_str.get(), m_cur_str.size());
+            mp_strings->append_segment(m_cur_str);
         break;
         case XML_si:
         {
@@ -226,7 +223,7 @@ bool xlsx_shared_strings_context::end_element(xmlns_id_t ns, xml_token_t name)
             else
             {
                 // unformatted text should only have one text segment.
-                mp_strings->append(m_cur_str.get(), m_cur_str.size());
+                mp_strings->append(m_cur_str);
             }
         }
         break;
@@ -234,9 +231,9 @@ bool xlsx_shared_strings_context::end_element(xmlns_id_t ns, xml_token_t name)
     return pop_stack(ns, name);
 }
 
-void xlsx_shared_strings_context::characters(const pstring& str, bool transient)
+void xlsx_shared_strings_context::characters(std::string_view str, bool transient)
 {
-    xml_token_pair_t& cur_token = get_current_element();
+    xml_token_pair_t cur_token = get_current_element();
     if (cur_token.first == NS_ooxml_xlsx && cur_token.second == XML_t)
     {
         m_cur_str = str;
@@ -268,9 +265,7 @@ void xlsx_shared_strings_context::characters(const pstring& str, bool transient)
                 // Append the tail end.
                 m_cell_buffer.append(p0, std::distance(p0, p));
 
-            m_cur_str = m_pool.intern(
-                m_cell_buffer.get(), m_cell_buffer.size()).first;
-
+            m_cur_str = m_pool.intern({m_cell_buffer.get(), m_cell_buffer.size()}).first;
             transient = false;
         }
 
@@ -350,7 +345,7 @@ const map_type& get()
 
 }
 
-class border_attr_parser : public unary_function<xml_token_attr_t, void>
+class border_attr_parser
 {
     spreadsheet::border_direction_t m_dir;
     spreadsheet::iface::import_styles& m_styles;
@@ -365,14 +360,14 @@ public:
             case XML_style:
             {
                 m_styles.set_border_style(m_dir,
-                    border_style::get().find(attr.value.get(), attr.value.size()));
+                    border_style::get().find(attr.value.data(), attr.value.size()));
             }
             break;
         }
     }
 };
 
-class cell_style_attr_parser : public unary_function<xml_token_attr_t, void>
+class cell_style_attr_parser
 {
     spreadsheet::iface::import_styles& m_styles;
 public:
@@ -384,7 +379,7 @@ public:
         switch (attr.name)
         {
             case XML_name:
-                m_styles.set_cell_style_name(attr.value.get(), attr.value.size());
+                m_styles.set_cell_style_name(attr.value);
             break;
             case XML_xfId:
             {
@@ -402,7 +397,7 @@ public:
     }
 };
 
-class xf_attr_parser : public unary_function<xml_token_attr_t, void>
+class xf_attr_parser
 {
     spreadsheet::iface::import_styles& m_styles;
 public:
@@ -461,7 +456,7 @@ public:
     }
 };
 
-class fill_color_attr_parser : public unary_function<xml_token_attr_t, void>
+class fill_color_attr_parser
 {
     spreadsheet::iface::import_styles& m_styles;
     const tokens& m_tokens;
@@ -500,7 +495,7 @@ public:
     }
 };
 
-class cell_protection_attr_parser : public unary_function<xml_token_attr_t, void>
+class cell_protection_attr_parser
 {
     spreadsheet::iface::import_styles& m_styles;
 public:
@@ -528,7 +523,7 @@ public:
     }
 };
 
-class cell_alignment_attr_parser : public unary_function<xml_token_attr_t, void>
+class cell_alignment_attr_parser
 {
     spreadsheet::hor_alignment_t m_hor_align;
     spreadsheet::ver_alignment_t m_ver_align;
@@ -598,11 +593,6 @@ xlsx_styles_context::xlsx_styles_context(session_context& session_cxt, const tok
 
 xlsx_styles_context::~xlsx_styles_context() {}
 
-bool xlsx_styles_context::can_handle_element(xmlns_id_t /*ns*/, xml_token_t /*name*/) const
-{
-    return true;
-}
-
 xml_context_base* xlsx_styles_context::create_child_context(xmlns_id_t /*ns*/, xml_token_t /*name*/)
 {
     return nullptr;
@@ -615,305 +605,327 @@ void xlsx_styles_context::end_child_context(xmlns_id_t /*ns*/, xml_token_t /*nam
 void xlsx_styles_context::start_element(xmlns_id_t ns, xml_token_t name, const xml_attrs_t& attrs)
 {
     xml_token_pair_t parent = push_stack(ns, name);
-    switch (name)
-    {
-        case XML_styleSheet:
-        {
-            // root element
-            xml_element_expected(parent, XMLNS_UNKNOWN_ID, XML_UNKNOWN_TOKEN);
-            if (get_config().debug)
-                print_attrs(get_tokens(), attrs);
-        }
-        break;
-        case XML_fonts:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
-            pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
-            size_t font_count = to_long(ps);
-            mp_styles->set_font_count(font_count);
-        }
-        break;
-        case XML_font:
-        {
-            xml_elem_stack_t expected_elements;
-            expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_fonts));
-            expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_dxf));
-            xml_element_expected(parent, expected_elements);
-        }
-        break;
-        case XML_b:
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
-            mp_styles->set_font_bold(true);
-        break;
-        case XML_i:
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
-            mp_styles->set_font_italic(true);
-        break;
-        case XML_u:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
-            pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_val)).get_value();
-            if (ps == "double")
-                mp_styles->set_font_underline(spreadsheet::underline_t::double_line);
-            else if (ps == "single")
-                mp_styles->set_font_underline(spreadsheet::underline_t::single_line);
-            else if (ps == "singleAccounting")
-                mp_styles->set_font_underline(spreadsheet::underline_t::single_accounting);
-            else if (ps == "doubleAccounting")
-                mp_styles->set_font_underline(spreadsheet::underline_t::double_accounting);
-        }
-        break;
-        case XML_sz:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
-            pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_val)).get_value();
-            double font_size = to_double(ps);
-            mp_styles->set_font_size(font_size);
-        }
-        break;
-        case XML_color:
-        {
-            // The color element can occur under various parent elements.
-            xml_elem_stack_t allowed;
-            allowed.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_font));
-            allowed.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_top));
-            allowed.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_bottom));
-            allowed.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_left));
-            allowed.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_right));
-            allowed.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_diagonal));
-            allowed.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_mruColors));
-            xml_element_expected(parent, allowed);
 
-            if (parent.first == NS_ooxml_xlsx)
+    if (ns == NS_ooxml_xlsx)
+    {
+        switch (name)
+        {
+            case XML_styleSheet:
             {
-                switch (parent.second)
+                // root element
+                xml_element_expected(parent, XMLNS_UNKNOWN_ID, XML_UNKNOWN_TOKEN);
+                if (get_config().debug)
+                    print_attrs(get_tokens(), attrs);
+            }
+            break;
+            case XML_fonts:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
+                pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
+                size_t font_count = to_long(ps);
+                mp_styles->set_font_count(font_count);
+            }
+            break;
+            case XML_font:
+            {
+                xml_elem_stack_t expected_elements;
+                expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_fonts));
+                expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_dxf));
+                xml_element_expected(parent, expected_elements);
+            }
+            break;
+            case XML_b:
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
+                mp_styles->set_font_bold(true);
+            break;
+            case XML_i:
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
+                mp_styles->set_font_italic(true);
+            break;
+            case XML_u:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
+                pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_val)).get_value();
+                if (ps == "double")
+                    mp_styles->set_font_underline(spreadsheet::underline_t::double_line);
+                else if (ps == "single")
+                    mp_styles->set_font_underline(spreadsheet::underline_t::single_line);
+                else if (ps == "singleAccounting")
+                    mp_styles->set_font_underline(spreadsheet::underline_t::single_accounting);
+                else if (ps == "doubleAccounting")
+                    mp_styles->set_font_underline(spreadsheet::underline_t::double_accounting);
+            }
+            break;
+            case XML_sz:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
+                pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_val)).get_value();
+                double font_size = to_double(ps);
+                mp_styles->set_font_size(font_size);
+            }
+            break;
+            case XML_color:
+            {
+                // The color element can occur under various parent elements.
+                const xml_elem_set_t expected = {
+                    { NS_ooxml_xlsx, XML_bottom },
+                    { NS_ooxml_xlsx, XML_diagonal },
+                    { NS_ooxml_xlsx, XML_end },
+                    { NS_ooxml_xlsx, XML_font },
+                    { NS_ooxml_xlsx, XML_horizontal },
+                    { NS_ooxml_xlsx, XML_left },
+                    { NS_ooxml_xlsx, XML_mruColors },
+                    { NS_ooxml_xlsx, XML_right },
+                    { NS_ooxml_xlsx, XML_start },
+                    { NS_ooxml_xlsx, XML_stop },
+                    { NS_ooxml_xlsx, XML_top },
+                    { NS_ooxml_xlsx, XML_vertical },
+                };
+                xml_element_expected(parent, expected);
+
+                if (parent.first == NS_ooxml_xlsx)
                 {
-                    case XML_top:
-                    case XML_bottom:
-                    case XML_left:
-                    case XML_right:
-                    case XML_diagonal:
-                        // This color is for a border.
-                        start_border_color(attrs);
-                    break;
-                    case XML_font:
-                        start_font_color(attrs);
-                    default:
-                        ;
+                    switch (parent.second)
+                    {
+                        case XML_top:
+                        case XML_bottom:
+                        case XML_left:
+                        case XML_right:
+                        case XML_diagonal:
+                            // This color is for a border.
+                            start_border_color(attrs);
+                        break;
+                        case XML_font:
+                            start_font_color(attrs);
+                        default:
+                            ;
+                    }
                 }
             }
-        }
-        break;
-        case XML_name:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
-            pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_val)).get_value();
-            mp_styles->set_font_name(ps.get(), ps.size());
-        }
-        break;
-        case XML_family:
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
-        break;
-        case XML_scheme:
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
-        break;
-        case XML_fills:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
-            pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
-            size_t fill_count = to_long(ps);
-            mp_styles->set_fill_count(fill_count);
-        }
-        break;
-        case XML_fill:
-        {
-            xml_elem_stack_t expected = {
-                xml_token_pair_t(NS_ooxml_xlsx, XML_fills),
-                xml_token_pair_t(NS_ooxml_xlsx, XML_dxf)
-            };
-
-            xml_element_expected(parent, expected);
-        }
-        break;
-        case XML_patternFill:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_fill);
-            pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_patternType)).get_value();
-            mp_styles->set_fill_pattern_type(fill_pattern::get().find(ps.data(), ps.size()));
-        }
-        break;
-        case XML_fgColor:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_patternFill);
-            for_each(
-                attrs.begin(), attrs.end(),
-                fill_color_attr_parser(
-                    *mp_styles, get_tokens(), true, get_config().debug));
-        }
-        break;
-        case XML_bgColor:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_patternFill);
-            for_each(
-                attrs.begin(), attrs.end(),
-                fill_color_attr_parser(
-                    *mp_styles, get_tokens(), false, get_config().debug));
-        }
-        break;
-        case XML_borders:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
-            pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
-            size_t border_count = to_long(ps);
-            mp_styles->set_border_count(border_count);
-        }
-        break;
-        case XML_border:
-        {
-            start_element_border(parent, attrs);
             break;
-        }
-        case XML_top:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
-            m_cur_border_dir = spreadsheet::border_direction_t::top;
-            border_attr_parser func(spreadsheet::border_direction_t::top, *mp_styles);
-            for_each(attrs.begin(), attrs.end(), func);
-        }
-        break;
-        case XML_bottom:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
-            m_cur_border_dir = spreadsheet::border_direction_t::bottom;
-            border_attr_parser func(spreadsheet::border_direction_t::bottom, *mp_styles);
-            for_each(attrs.begin(), attrs.end(), func);
-        }
-        break;
-        case XML_left:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
-            m_cur_border_dir = spreadsheet::border_direction_t::left;
-            border_attr_parser func(spreadsheet::border_direction_t::left, *mp_styles);
-            for_each(attrs.begin(), attrs.end(), func);
-        }
-        break;
-        case XML_right:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
-            m_cur_border_dir = spreadsheet::border_direction_t::right;
-            border_attr_parser func(spreadsheet::border_direction_t::right, *mp_styles);
-            for_each(attrs.begin(), attrs.end(), func);
-        }
-        break;
-        case XML_diagonal:
-        {
-            start_element_diagonal(parent, attrs);
+            case XML_name:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
+                std::string_view ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_val)).get_value();
+                mp_styles->set_font_name(ps);
+            }
             break;
-        }
-        case XML_cellStyleXfs:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
-            pstring ps = for_each(
-                attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
-            size_t n = strtoul(ps.get(), nullptr, 10);
-            mp_styles->set_cell_style_xf_count(n);
-            m_cell_style_xf = true;
-        }
-        break;
-        case XML_cellXfs:
-        {
-            // Collection of un-named cell formats used in the document.
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
-            pstring ps = for_each(
-                attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
-            size_t n = strtoul(ps.get(), nullptr, 10);
-            mp_styles->set_cell_xf_count(n);
-            m_cell_style_xf = false;
-        }
-        break;
-        case XML_dxfs:
-        {
-            // Collection of differential formats used in the document.
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
-            pstring ps = for_each(
-                attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
-            size_t n = strtoul(ps.get(), nullptr, 10);
-            mp_styles->set_dxf_count(n);
-        }
-        break;
-        case XML_cellStyles:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
-            pstring ps = for_each(
-                attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
-            size_t n = strtoul(ps.get(), nullptr, 10);
-            mp_styles->set_cell_style_count(n);
-        }
-        break;
-        case XML_cellStyle:
-        {
-            // named cell style, some of which are built-in such as 'Normal'.
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_cellStyles);
-            for_each(attrs.begin(), attrs.end(), cell_style_attr_parser(*mp_styles));
-        }
-        break;
-        case XML_xf:
-        {
-            // Actual cell format attributes (for some reason) abbreviated to
-            // 'xf'.  Used both by cells and cell styles.
-            xml_elem_stack_t allowed;
-            allowed.push_back(xml_elem_stack_t::value_type(NS_ooxml_xlsx, XML_cellXfs));
-            allowed.push_back(xml_elem_stack_t::value_type(NS_ooxml_xlsx, XML_cellStyleXfs));
-            xml_element_expected(parent, allowed);
+            case XML_family:
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
+            break;
+            case XML_scheme:
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_font);
+            break;
+            case XML_fills:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
+                pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
+                size_t fill_count = to_long(ps);
+                mp_styles->set_fill_count(fill_count);
+            }
+            break;
+            case XML_fill:
+            {
+                xml_elem_stack_t expected = {
+                    xml_token_pair_t(NS_ooxml_xlsx, XML_fills),
+                    xml_token_pair_t(NS_ooxml_xlsx, XML_dxf)
+                };
 
-            for_each(attrs.begin(), attrs.end(), xf_attr_parser(*mp_styles));
-        }
-        break;
-        case XML_dxf:
-        break;
-        case XML_protection:
-        {
-            xml_elem_stack_t expected_elements;
-            expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_xf));
-            expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_dxf));
-            xml_element_expected(parent, expected_elements);
-            for_each(attrs.begin(), attrs.end(), cell_protection_attr_parser(*mp_styles));
-        }
-        break;
-        case XML_alignment:
-        {
-            xml_elem_stack_t expected_elements;
-            expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_xf));
-            expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_dxf));
-            xml_element_expected(parent, expected_elements);
-            cell_alignment_attr_parser func;
-            func = for_each(attrs.begin(), attrs.end(), func);
-            mp_styles->set_xf_horizontal_alignment(func.get_hor_align());
-            mp_styles->set_xf_vertical_alignment(func.get_ver_align());
-        }
-        break;
-        case XML_numFmts:
-        {
-            xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
-            pstring val =
+                xml_element_expected(parent, expected);
+            }
+            break;
+            case XML_patternFill:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_fill);
+                pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_patternType)).get_value();
+                mp_styles->set_fill_pattern_type(fill_pattern::get().find(ps.data(), ps.size()));
+            }
+            break;
+            case XML_fgColor:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_patternFill);
                 for_each(
                     attrs.begin(), attrs.end(),
-                    single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
-            size_t n = to_long(val);
-            mp_styles->set_number_format_count(n);
+                    fill_color_attr_parser(
+                        *mp_styles, get_tokens(), true, get_config().debug));
+            }
+            break;
+            case XML_bgColor:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_patternFill);
+                for_each(
+                    attrs.begin(), attrs.end(),
+                    fill_color_attr_parser(
+                        *mp_styles, get_tokens(), false, get_config().debug));
+            }
+            break;
+            case XML_borders:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
+                pstring ps = for_each(attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
+                size_t border_count = to_long(ps);
+                mp_styles->set_border_count(border_count);
+            }
+            break;
+            case XML_border:
+            {
+                start_element_border(parent, attrs);
+                break;
+            }
+            case XML_top:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
+                m_cur_border_dir = spreadsheet::border_direction_t::top;
+                border_attr_parser func(spreadsheet::border_direction_t::top, *mp_styles);
+                for_each(attrs.begin(), attrs.end(), func);
+            }
+            break;
+            case XML_bottom:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
+                m_cur_border_dir = spreadsheet::border_direction_t::bottom;
+                border_attr_parser func(spreadsheet::border_direction_t::bottom, *mp_styles);
+                for_each(attrs.begin(), attrs.end(), func);
+            }
+            break;
+            case XML_left:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
+                m_cur_border_dir = spreadsheet::border_direction_t::left;
+                border_attr_parser func(spreadsheet::border_direction_t::left, *mp_styles);
+                for_each(attrs.begin(), attrs.end(), func);
+            }
+            break;
+            case XML_right:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_border);
+                m_cur_border_dir = spreadsheet::border_direction_t::right;
+                border_attr_parser func(spreadsheet::border_direction_t::right, *mp_styles);
+                for_each(attrs.begin(), attrs.end(), func);
+            }
+            break;
+            case XML_diagonal:
+            {
+                start_element_diagonal(parent, attrs);
+                break;
+            }
+            case XML_cellStyleXfs:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
+                pstring ps = for_each(
+                    attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
+                if (!ps.empty())
+                {
+                    size_t n = strtoul(ps.data(), nullptr, 10);
+                    mp_styles->set_cell_style_xf_count(n);
+                }
+                m_cell_style_xf = true;
+            }
+            break;
+            case XML_cellXfs:
+            {
+                // Collection of un-named cell formats used in the document.
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
+                pstring ps = for_each(
+                    attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
+                if (!ps.empty())
+                {
+                    size_t n = strtoul(ps.data(), nullptr, 10);
+                    mp_styles->set_cell_xf_count(n);
+                }
+                m_cell_style_xf = false;
+            }
+            break;
+            case XML_dxfs:
+            {
+                // Collection of differential formats used in the document.
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
+                pstring ps = for_each(
+                    attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
+                if (!ps.empty())
+                {
+                    size_t n = strtoul(ps.data(), nullptr, 10);
+                    mp_styles->set_dxf_count(n);
+                }
+            }
+            break;
+            case XML_cellStyles:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
+                pstring ps = for_each(
+                    attrs.begin(), attrs.end(), single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
+                if (!ps.empty())
+                {
+                    size_t n = strtoul(ps.data(), nullptr, 10);
+                    mp_styles->set_cell_style_count(n);
+                }
+            }
+            break;
+            case XML_cellStyle:
+            {
+                // named cell style, some of which are built-in such as 'Normal'.
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_cellStyles);
+                for_each(attrs.begin(), attrs.end(), cell_style_attr_parser(*mp_styles));
+            }
+            break;
+            case XML_xf:
+            {
+                // Actual cell format attributes (for some reason) abbreviated to
+                // 'xf'.  Used both by cells and cell styles.
+                xml_elem_stack_t allowed;
+                allowed.push_back(xml_elem_stack_t::value_type(NS_ooxml_xlsx, XML_cellXfs));
+                allowed.push_back(xml_elem_stack_t::value_type(NS_ooxml_xlsx, XML_cellStyleXfs));
+                xml_element_expected(parent, allowed);
+
+                for_each(attrs.begin(), attrs.end(), xf_attr_parser(*mp_styles));
+            }
+            break;
+            case XML_dxf:
+            break;
+            case XML_protection:
+            {
+                xml_elem_stack_t expected_elements;
+                expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_xf));
+                expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_dxf));
+                xml_element_expected(parent, expected_elements);
+                for_each(attrs.begin(), attrs.end(), cell_protection_attr_parser(*mp_styles));
+            }
+            break;
+            case XML_alignment:
+            {
+                xml_elem_stack_t expected_elements;
+                expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_xf));
+                expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_dxf));
+                xml_element_expected(parent, expected_elements);
+                cell_alignment_attr_parser func;
+                func = for_each(attrs.begin(), attrs.end(), func);
+                mp_styles->set_xf_horizontal_alignment(func.get_hor_align());
+                mp_styles->set_xf_vertical_alignment(func.get_ver_align());
+            }
+            break;
+            case XML_numFmts:
+            {
+                xml_element_expected(parent, NS_ooxml_xlsx, XML_styleSheet);
+                pstring val =
+                    for_each(
+                        attrs.begin(), attrs.end(),
+                        single_attr_getter(m_pool, NS_ooxml_xlsx, XML_count)).get_value();
+                if (!val.empty())
+                {
+                    size_t n = to_long(val);
+                    mp_styles->set_number_format_count(n);
+                }
+            }
+            break;
+            case XML_numFmt:
+                start_element_number_format(parent, attrs);
+                break;
+            default:
+                warn_unhandled();
         }
-        break;
-        case XML_numFmt:
-        {
-            xml_elem_stack_t expected_elements;
-            expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_numFmts));
-            expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_dxf));
-            xml_element_expected(parent, expected_elements);
-        }
-        break;
-        default:
-            warn_unhandled();
     }
+    else
+        warn_unhandled();
 }
 
 bool xlsx_styles_context::end_element(xmlns_id_t ns, xml_token_t name)
@@ -945,17 +957,52 @@ bool xlsx_styles_context::end_element(xmlns_id_t ns, xml_token_t name)
         {
             size_t id = mp_styles->commit_cell_protection();
             mp_styles->set_xf_protection(id);
+            break;
         }
-        break;
+        case XML_numFmt:
+            end_element_number_format();
+            break;
     }
     return pop_stack(ns, name);
 }
 
-void xlsx_styles_context::characters(const pstring& /*str*/, bool /*transient*/)
+void xlsx_styles_context::characters(std::string_view /*str*/, bool /*transient*/)
 {
     // not used in the styles.xml part.
 }
 
+void xlsx_styles_context::start_element_number_format(const xml_token_pair_t& parent, const xml_attrs_t& attrs)
+{
+    xml_elem_stack_t expected_elements;
+    expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_numFmts));
+    expected_elements.push_back(xml_token_pair_t(NS_ooxml_xlsx, XML_dxf));
+    xml_element_expected(parent, expected_elements);
+
+    if (mp_styles)
+    {
+        for (const xml_token_attr_t& attr : attrs)
+        {
+            if (attr.ns && attr.ns != NS_ooxml_xlsx)
+                continue;
+
+            switch (attr.name)
+            {
+                case XML_numFmtId:
+                {
+                    long id = to_long(attr.value);
+                    mp_styles->set_number_format_identifier(id);
+                    break;
+                }
+                case XML_formatCode:
+                {
+                    mp_styles->set_number_format_code(attr.value);
+                    break;
+                }
+            }
+        }
+    }
+}
+
 void xlsx_styles_context::start_element_border(const xml_token_pair_t& parent, const xml_attrs_t& attrs)
 {
     xml_elem_stack_t expected_elements;
@@ -969,7 +1016,7 @@ void xlsx_styles_context::start_element_border(const xml_token_pair_t& parent, c
     std::for_each(attrs.begin(), attrs.end(),
         [&diagonal_up,&diagonal_down](const xml_token_attr_t& attr)
         {
-            if (attr.ns != NS_ooxml_xlsx)
+            if (attr.ns && attr.ns != NS_ooxml_xlsx)
                 return;
 
             switch (attr.name)
@@ -1044,5 +1091,14 @@ void xlsx_styles_context::start_font_color(const xml_attrs_t& attrs)
         mp_styles->set_font_color(alpha, red, green, blue);
 }
 
+void xlsx_styles_context::end_element_number_format()
+{
+    if (!mp_styles)
+        return;
+
+    mp_styles->commit_number_format();
 }
+
+}
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
