@@ -118,7 +118,7 @@ xml_map_tree::element* xml_map_tree::element::get_or_create_child(
     string_pool& sp = parent.m_names;
 
     // Insert a new element of this name.
-    auto const nm = sp.intern(_name.name.get(), _name.name.size()).first; // work around LLVM < 7 libc++ bug
+    auto const nm = sp.intern(_name.name).first; // work around LLVM < 7 libc++ bug
     child_elements->push_back(
         parent.m_element_pool.construct(
             element::args_type(
@@ -171,7 +171,7 @@ xml_map_tree::element* xml_map_tree::element::get_or_create_linked_child(
     string_pool& sp = parent.m_names;
 
     // Insert a new linked element of this name.
-    auto const nm = sp.intern(_name.name.get(), _name.name.size()).first; // work around LLVM < 7 libc++ bug
+    auto const nm = sp.intern(_name.name).first; // work around LLVM < 7 libc++ bug
     child_elements->push_back(
         parent.m_element_pool.construct(
             element::args_type(
@@ -589,7 +589,7 @@ xml_map_tree::range_reference* xml_map_tree::get_range_reference(const cell_posi
 
         // Make sure the sheet name string is persistent.
         cell_position pos_safe = pos;
-        pos_safe.sheet = m_names.intern(pos.sheet.get(), pos.sheet.size()).first;
+        pos_safe.sheet = m_names.intern(pos.sheet).first;
 
         it = m_field_refs.insert(
             it, range_ref_map_type::value_type(
@@ -690,7 +690,7 @@ xml_map_tree::linked_node_type xml_map_tree::get_linked_node(const pstring& xpat
         if (it != attrs.end())
             throw xpath_error("This attribute is already linked.  You can't link the same attribute twice.");
 
-        auto const nm = m_names.intern(token.name.get(), token.name.size()).first; // work around LLVM < 7 libc++ bug
+        auto const nm = m_names.intern(token.name).first; // work around LLVM < 7 libc++ bug
         attribute* p = m_attribute_pool.construct(
             attribute::args_type(
                 *this,
