@@ -5,14 +5,15 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include "orcus/xml_structure_tree.hpp"
-#include "orcus/sax_ns_parser.hpp"
-#include "orcus/xml_namespace.hpp"
-#include "orcus/global.hpp"
-#include "orcus/exception.hpp"
+#include <orcus/xml_structure_tree.hpp>
+#include <orcus/sax_ns_parser.hpp>
+#include <orcus/xml_namespace.hpp>
+#include <orcus/global.hpp>
+#include <orcus/exception.hpp>
+#include <orcus/string_pool.hpp>
 
-#include "orcus/string_pool.hpp"
 #include "string_helper.hpp"
+#include "xml_structure_mapper.hpp"
 
 #include <iostream>
 #include <sstream>
@@ -22,8 +23,6 @@
 #include <unordered_map>
 #include <unordered_set>
 
-using namespace std;
-
 namespace orcus {
 
 namespace {
@@ -55,11 +54,23 @@ struct elem_prop
      */
     bool repeat;
 
+    bool has_content;
+
     elem_prop(const elem_prop&) = delete;
     elem_prop& operator=(const elem_prop&) = delete;
 
-    elem_prop() : appearance_order(0), in_scope_count(1), repeat(false) {}
-    elem_prop(size_t _appearance_order) : appearance_order(_appearance_order), in_scope_count(1), repeat(false) {}
+    elem_prop() :
+        appearance_order(0),
+        in_scope_count(1),
+        repeat(false),
+        has_content(false) {}
+
+    elem_prop(size_t _appearance_order) :
+        appearance_order(_appearance_order),
+        in_scope_count(1),
+        repeat(false),
+        has_content(false) {}
+
     ~elem_prop()
     {
         for_each(child_elements.begin(), child_elements.end(), map_object_deleter<element_store_type>());
@@ -114,11 +125,11 @@ public:
 
     void doctype(const sax::doctype_declaration&) {}
 
-    void start_declaration(const pstring& name)
+    void start_declaration(const pstring& /*name*/)
     {
     }
 
-    void end_declaration(const pstring& name)
+    void end_declaration(const pstring& /*name*/)
     {
         m_attrs.clear();
     }
@@ -159,7 +170,7 @@ public:
         // New element.
         size_t order = current.prop->child_elements.size();
         key.name = m_pool.intern(key.name).first;
-        pair<element_store_type::const_iterator,bool> r =
+        std::pair<element_store_type::const_iterator,bool> r =
             current.prop->child_elements.insert(
                 element_store_type::value_type(key, new elem_prop(order)));
 
@@ -174,7 +185,7 @@ public:
         m_stack.push_back(ref);
     }
 
-    void end_element(const sax_ns_parser_element& elem)
+    void end_element(const sax_ns_parser_element& /*elem*/)
     {
         if (m_stack.empty())
             throw general_error("Element stack is empty.");
@@ -190,7 +201,14 @@ public:
         m_stack.pop_back();
     }
 
-    void characters(const pstring&, bool) {}
+    void characters(std::string_view, bool)
+    {
+        if (m_stack.empty())
+            return;
+
+        element_ref& current = m_stack.back();
+        current.prop->has_content = true;
+    }
 
     void attribute(const pstring&, const pstring&)
     {
@@ -208,7 +226,7 @@ public:
     }
 };
 
-struct sort_by_appearance : std::binary_function<element_ref, element_ref, bool>
+struct sort_by_appearance
 {
     bool operator() (const element_ref& left, const element_ref& right) const
     {
@@ -239,7 +257,7 @@ struct scope
 
 typedef std::vector<std::unique_ptr<scope>> scopes_type;
 
-void print_scope(ostream& os, const scopes_type& scopes, const xmlns_context& cxt)
+void print_scope(std::ostream& os, const scopes_type& scopes, const xmlns_context& cxt)
 {
     if (scopes.empty())
         throw general_error("scope stack shouldn't be empty while dumping tree.");
@@ -260,6 +278,10 @@ void print_scope(ostream& os, const scopes_type& scopes, const xmlns_context& cx
 
 }
 
+xml_table_range_t::xml_table_range_t() {}
+
+xml_table_range_t::~xml_table_range_t() {}
+
 struct xml_structure_tree::impl
 {
     string_pool m_pool;
@@ -272,9 +294,9 @@ struct xml_structure_tree::impl
     impl(xmlns_context& xmlns_cxt) : m_xmlns_cxt(xmlns_cxt) {}
     ~impl() {}
 
-    std::string get_element_str(const xml_structure_tree::entity_name& name) const
+    std::string to_string(const xml_structure_tree::entity_name& name) const
     {
-        ostringstream ss;
+        std::ostringstream ss;
         if (m_xmlns_cxt.get_index(name.ns) != index_not_found)
             ss << m_xmlns_cxt.get_short_name(name.ns) << ":";
         ss << name.name;
@@ -301,7 +323,7 @@ struct xml_structure_tree::walker_impl
 xml_structure_tree::entity_name::entity_name() :
     ns(XMLNS_UNKNOWN_ID) {}
 
-xml_structure_tree::entity_name::entity_name(xmlns_id_t _ns, const pstring& _name) :
+xml_structure_tree::entity_name::entity_name(xmlns_id_t _ns, std::string_view _name) :
     ns(_ns), name(_name) {}
 
 bool xml_structure_tree::entity_name::operator< (const entity_name& r) const
@@ -326,18 +348,18 @@ size_t xml_structure_tree::entity_name::hash::operator() (const entity_name& val
 }
 
 xml_structure_tree::element::element() :
-    repeat(false) {}
+    repeat(false), has_content(false) {}
 
-xml_structure_tree::element::element(const entity_name& _name, bool _repeat) :
-    name(_name), repeat(_repeat) {}
+xml_structure_tree::element::element(const entity_name& _name, bool _repeat, bool _has_content) :
+    name(_name), repeat(_repeat), has_content(_has_content) {}
 
 xml_structure_tree::walker::walker(const xml_structure_tree::impl& parent_impl) :
-    mp_impl(orcus::make_unique<walker_impl>(parent_impl))
+    mp_impl(std::make_unique<walker_impl>(parent_impl))
 {
 }
 
 xml_structure_tree::walker::walker(const walker& r) :
-    mp_impl(orcus::make_unique<walker_impl>(*r.mp_impl))
+    mp_impl(std::make_unique<walker_impl>(*r.mp_impl))
 {
 }
 
@@ -360,7 +382,7 @@ xml_structure_tree::element xml_structure_tree::walker::root()
     element_ref ref(mp_impl->mp_root->name, &mp_impl->mp_root->prop);
     mp_impl->m_cur_elem = ref;
     mp_impl->m_scopes.push_back(ref);
-    return xml_structure_tree::element(ref.name, false);
+    return xml_structure_tree::element(ref.name, false, ref.prop->has_content);
 }
 
 xml_structure_tree::element xml_structure_tree::walker::descend(const entity_name& name)
@@ -379,7 +401,7 @@ xml_structure_tree::element xml_structure_tree::walker::descend(const entity_nam
     element_ref ref(name, it->second);
     mp_impl->m_scopes.push_back(ref);
 
-    return element(name, it->second->repeat);
+    return element(name, it->second->repeat, it->second->has_content);
 }
 
 xml_structure_tree::element xml_structure_tree::walker::ascend()
@@ -392,27 +414,31 @@ xml_structure_tree::element xml_structure_tree::walker::ascend()
 
     mp_impl->m_scopes.pop_back();
     const element_ref& ref = mp_impl->m_scopes.back();
-    return element(ref.name, ref.prop->repeat);
+    return element(ref.name, ref.prop->repeat, ref.prop->has_content);
 }
 
-void xml_structure_tree::walker::get_children(entity_names_type& names)
+xml_structure_tree::entity_names_type xml_structure_tree::walker::get_children()
 {
     if (mp_impl->m_scopes.empty())
         throw general_error("Scope is empty.");
 
+    entity_names_type names;
     assert(mp_impl->m_scopes.back().prop);
     const elem_prop& prop = *mp_impl->m_scopes.back().prop;
     names.assign(prop.child_element_names.begin(), prop.child_element_names.end());
+    return names;
 }
 
-void xml_structure_tree::walker::get_attributes(entity_names_type& names)
+xml_structure_tree::entity_names_type xml_structure_tree::walker::get_attributes()
 {
     if (mp_impl->m_scopes.empty())
         throw general_error("Scope is empty.");
 
+    entity_names_type names;
     assert(mp_impl->m_scopes.back().prop);
     const elem_prop& prop = *mp_impl->m_scopes.back().prop;
     names.assign(prop.attribute_names.begin(), prop.attribute_names.end());
+    return names;
 }
 
 size_t xml_structure_tree::walker::get_xmlns_index(xmlns_id_t ns) const
@@ -420,23 +446,28 @@ size_t xml_structure_tree::walker::get_xmlns_index(xmlns_id_t ns) const
     return mp_impl->m_parent_impl.m_xmlns_cxt.get_index(ns);
 }
 
-string xml_structure_tree::walker::get_xmlns_short_name(xmlns_id_t ns) const
+std::string xml_structure_tree::walker::get_xmlns_short_name(xmlns_id_t ns) const
 {
     return mp_impl->m_parent_impl.m_xmlns_cxt.get_short_name(ns);
 }
 
-string xml_structure_tree::walker::get_path() const
+std::string xml_structure_tree::walker::to_string(const entity_name& name) const
 {
-    ostringstream ss;
+    return mp_impl->m_parent_impl.to_string(name);
+}
+
+std::string xml_structure_tree::walker::get_path() const
+{
+    std::ostringstream ss;
     for (auto& element : mp_impl->m_scopes)
     {
-        ss << "/" << mp_impl->m_parent_impl.get_element_str(element.name);
+        ss << "/" << mp_impl->m_parent_impl.to_string(element.name);
     }
 
     return ss.str();
 }
 
-xml_structure_tree::element xml_structure_tree::walker::select_by_path(const std::string& path)
+xml_structure_tree::element xml_structure_tree::walker::move_to(const std::string& path)
 {
     pstring p(path);
     std::vector<pstring> parts = string_helper::split_string(p, '/');
@@ -457,7 +488,7 @@ xml_structure_tree::element xml_structure_tree::walker::select_by_path(const std
         throw general_error("invalid format for path");
 
     element_ref root_ref(mp_impl->mp_root->name, &mp_impl->mp_root->prop);
-    if (pstring(mp_impl->m_parent_impl.get_element_str(root_ref.name)) != parts[0])
+    if (pstring(mp_impl->m_parent_impl.to_string(root_ref.name)) != parts[0])
         throw general_error("path does not match any element");
 
     std::vector<element_ref> scopes;
@@ -469,7 +500,7 @@ xml_structure_tree::element xml_structure_tree::walker::select_by_path(const std
         bool found = false;
         for (auto& child : prop.child_elements)
         {
-            if (pstring(mp_impl->m_parent_impl.get_element_str(child.first)) == parts[i])
+            if (pstring(mp_impl->m_parent_impl.to_string(child.first)) == parts[i])
             {
                 scopes.emplace_back(child.first, child.second);
                 found = true;
@@ -482,23 +513,29 @@ xml_structure_tree::element xml_structure_tree::walker::select_by_path(const std
 
     std::swap(mp_impl->m_scopes, scopes);
     const element_ref& ref = mp_impl->m_scopes.back();
-    return element(ref.name, ref.prop->repeat);
+    return element(ref.name, ref.prop->repeat, ref.prop->has_content);
 }
 
 xml_structure_tree::xml_structure_tree(xmlns_context& xmlns_cxt) :
-    mp_impl(orcus::make_unique<impl>(xmlns_cxt)) {}
+    mp_impl(std::make_unique<impl>(xmlns_cxt)) {}
+
+xml_structure_tree::xml_structure_tree(xml_structure_tree&& other) :
+    mp_impl(std::move(other.mp_impl))
+{
+    other.mp_impl = std::make_unique<impl>(mp_impl->m_xmlns_cxt);
+}
 
 xml_structure_tree::~xml_structure_tree() {}
 
-void xml_structure_tree::parse(const char* p, size_t n)
+void xml_structure_tree::parse(std::string_view s)
 {
     xml_sax_handler hdl(mp_impl->m_pool);
-    sax_ns_parser<xml_sax_handler> parser(p, n, mp_impl->m_xmlns_cxt, hdl);
+    sax_ns_parser<xml_sax_handler> parser(s.data(), s.size(), mp_impl->m_xmlns_cxt, hdl);
     parser.parse();
     mp_impl->mp_root = hdl.release_root_element();
 }
 
-void xml_structure_tree::dump_compact(ostream& os) const
+void xml_structure_tree::dump_compact(std::ostream& os) const
 {
     if (!mp_impl->mp_root)
         return;
@@ -510,7 +547,7 @@ void xml_structure_tree::dump_compact(ostream& os) const
     cxt.dump(os);
 
     element_ref ref(mp_impl->mp_root->name, &mp_impl->mp_root->prop);
-    scopes.push_back(orcus::make_unique<scope>(entity_name(), false, ref));
+    scopes.push_back(std::make_unique<scope>(entity_name(), false, ref));
     while (!scopes.empty())
     {
         bool new_scope = false;
@@ -520,7 +557,7 @@ void xml_structure_tree::dump_compact(ostream& os) const
         for (; cur_scope.current_pos != cur_scope.elements.end(); ++cur_scope.current_pos)
         {
             const element_ref& this_elem = *cur_scope.current_pos;
-            ostringstream ss;
+            std::ostringstream ss;
             print_scope(ss, scopes, cxt);
 
             ss << "/";
@@ -531,16 +568,12 @@ void xml_structure_tree::dump_compact(ostream& os) const
             if (this_elem.prop->repeat)
                 ss << "[*]";
 
-            string elem_name = ss.str();
-            os << elem_name << endl;
+            std::string elem_name = ss.str();
+            os << elem_name << std::endl;
 
             // Print all attributes that belong to this element.
-            {
-                const entity_names_type& attrs = this_elem.prop->attribute_names;
-                entity_names_type::const_iterator it = attrs.begin(), it_end = attrs.end();
-                for (; it != it_end; ++it)
-                    os << elem_name << '@' << it->name << endl;
-            }
+            for (const entity_name& attr : this_elem.prop->attribute_names)
+                os << elem_name << "/@" << mp_impl->to_string(attr) << std::endl;
 
             const element_store_type& child_elements = this_elem.prop->child_elements;
             if (child_elements.empty())
@@ -548,12 +581,11 @@ void xml_structure_tree::dump_compact(ostream& os) const
 
             // This element has child elements.  Push a new scope and populate
             // it with all child elements.
-            element_store_type::const_iterator it = child_elements.begin(), it_end = child_elements.end();
             elements_type elems;
-            for (; it != it_end; ++it)
+            for (const auto& entry : child_elements)
             {
-                ref.name = it->first;
-                ref.prop = it->second;
+                ref.name = entry.first;
+                ref.prop = entry.second;
                 elems.push_back(ref);
             }
 
@@ -564,7 +596,7 @@ void xml_structure_tree::dump_compact(ostream& os) const
 
             // Push a new scope, and restart the loop with the new scope.
             ++cur_scope.current_pos;
-            scopes.push_back(orcus::make_unique<scope>(this_elem.name, this_elem.prop->repeat));
+            scopes.push_back(std::make_unique<scope>(this_elem.name, this_elem.prop->repeat));
             scope& child_scope = *scopes.back();
             child_scope.elements.swap(elems);
             child_scope.current_pos = child_scope.elements.begin();
@@ -585,5 +617,17 @@ xml_structure_tree::walker xml_structure_tree::get_walker() const
     return walker(*mp_impl);
 }
 
+void xml_structure_tree::process_ranges(range_handler_type rh) const
+{
+    detail::xml_structure_mapper mapper(rh, get_walker());
+    mapper.run();
 }
+
+void xml_structure_tree::swap(xml_structure_tree& other)
+{
+    mp_impl.swap(other.mp_impl);
+}
+
+}
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
