@@ -11,9 +11,11 @@
 
 #include "orcus_filter_global.hpp"
 
+#include <iostream>
+
 using namespace orcus;
 
-int main(int argc, char** argv)
+int main(int argc, char** argv) try
 {
     spreadsheet::range_size_t ss{1048576, 16384};
     spreadsheet::document doc{ss};
@@ -25,4 +27,10 @@ int main(int argc, char** argv)
 
     return EXIT_SUCCESS;
 }
+catch (const std::exception& e)
+{
+    std::cerr << e.what() << std::endl;
+    return EXIT_FAILURE;
+}
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
