@@ -6,7 +6,7 @@
  */
 
 #include "orcus/orcus_ods.hpp"
-#include "orcus/pstring.hpp"
+#include "pstring.hpp"
 #include "orcus/global.hpp"
 #include "orcus/stream.hpp"
 #include "orcus/spreadsheet/factory.hpp"
@@ -36,6 +36,9 @@ std::vector<const char*> dirs = {
     SRCDIR"/test/ods/raw-values-1/",
     SRCDIR"/test/ods/formula-1/",
     SRCDIR"/test/ods/formula-2/",
+    SRCDIR"/test/ods/named-range/",
+    SRCDIR"/test/ods/named-expression/",
+    SRCDIR"/test/ods/named-expression-sheet-local/",
 };
 
 void test_ods_import_cell_values()
@@ -47,10 +50,12 @@ void test_ods_import_cell_values()
 
         // Read the input.ods document.
         path.append("input.ods");
-        spreadsheet::document doc;
+        spreadsheet::range_size_t ss{1048576, 16384};
+        spreadsheet::document doc{ss};
         spreadsheet::import_factory factory(doc);
         orcus_ods app(&factory);
         app.read_file(path.c_str());
+        doc.recalc_formula_cells();
 
         // Dump the content of the model.
         ostringstream os;
@@ -74,12 +79,13 @@ void test_ods_import_cell_values()
 void test_ods_import_column_widths_row_heights()
 {
     const char* filepath = SRCDIR"/test/ods/column-width-row-height/input.ods";
-    document doc;
+    spreadsheet::range_size_t ss{1048576, 16384};
+    document doc{ss};
     import_factory factory(doc);
     orcus_ods app(&factory);
     app.read_file(filepath);
 
-    assert(doc.sheet_size() > 0);
+    assert(doc.get_sheet_count() > 0);
     spreadsheet::sheet* sh = doc.get_sheet(0);
     assert(sh);
 
@@ -105,12 +111,12 @@ void test_ods_import_column_widths_row_heights()
 void test_ods_import_formatted_text()
 {
     const char* filepath = SRCDIR"/test/ods/formatted-text/bold-and-italic.ods";
-    document doc;
+    document doc{{1048576, 16384}};
     import_factory factory(doc);
     orcus_ods app(&factory);
     app.read_file(filepath);
 
-    assert(doc.sheet_size() > 0);
+    assert(doc.get_sheet_count() > 0);
     spreadsheet::sheet* sh = doc.get_sheet(0);
     assert(sh);
 
