@@ -5,8 +5,9 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
+#include "orcus_test_global.hpp"
 #include "orcus/orcus_xls_xml.hpp"
-#include "orcus/pstring.hpp"
+#include "pstring.hpp"
 #include "orcus/global.hpp"
 #include "orcus/stream.hpp"
 #include "orcus/config.hpp"
@@ -21,18 +22,19 @@
 
 #include <ixion/model_context.hpp>
 #include <ixion/address.hpp>
+#include <ixion/cell.hpp>
 
-#include <cassert>
 #include <string>
 #include <sstream>
 #include <vector>
 #include <cmath>
 #include <iostream>
+#include <fstream>
 
-#include "orcus_test_global.hpp"
 
 using namespace orcus;
 using namespace std;
+namespace ss = orcus::spreadsheet;
 
 namespace {
 
@@ -40,25 +42,58 @@ config test_config(format_t::xls_xml);
 
 std::vector<const char*> dirs = {
     SRCDIR"/test/xls-xml/basic/",
+    SRCDIR"/test/xls-xml/basic-utf-16-be/",
+    SRCDIR"/test/xls-xml/basic-utf-16-le/",
     SRCDIR"/test/xls-xml/bold-and-italic/",
     SRCDIR"/test/xls-xml/colored-text/",
     SRCDIR"/test/xls-xml/empty-rows/",
     SRCDIR"/test/xls-xml/formula-array-1/",
     SRCDIR"/test/xls-xml/formula-cells-1/",
     SRCDIR"/test/xls-xml/formula-cells-2/",
+    SRCDIR"/test/xls-xml/formula-cells-3/",
+    SRCDIR"/test/xls-xml/invalid-sub-structure/",
     SRCDIR"/test/xls-xml/leading-whitespace/",
     SRCDIR"/test/xls-xml/merged-cells/",
+    SRCDIR"/test/xls-xml/named-colors/",
     SRCDIR"/test/xls-xml/named-expression/",
     SRCDIR"/test/xls-xml/named-expression-sheet-local/",
     SRCDIR"/test/xls-xml/raw-values-1/",
+    SRCDIR"/test/xls-xml/table-offset/",
 };
 
-std::unique_ptr<spreadsheet::document> load_doc(const string& path)
+std::unique_ptr<spreadsheet::document> load_doc_from_filepath(
+    const std::string& path,
+    bool recalc=true,
+    ss::formula_error_policy_t error_policy=ss::formula_error_policy_t::fail)
 {
-    std::unique_ptr<spreadsheet::document> doc = orcus::make_unique<spreadsheet::document>();
+    spreadsheet::range_size_t ss{1048576, 16384};
+    std::unique_ptr<spreadsheet::document> doc = std::make_unique<spreadsheet::document>(ss);
+    spreadsheet::import_factory factory(*doc);
+    factory.set_recalc_formula_cells(recalc);
+    factory.set_formula_error_policy(error_policy);
+    orcus_xls_xml app(&factory);
+    app.set_config(test_config);
+    app.read_file(path.c_str());
+
+    return doc;
+}
+
+std::unique_ptr<spreadsheet::document> load_doc_from_stream(const string& path)
+{
+    spreadsheet::range_size_t ss{1048576, 16384};
+    std::unique_ptr<spreadsheet::document> doc = std::make_unique<spreadsheet::document>(ss);
     spreadsheet::import_factory factory(*doc);
     orcus_xls_xml app(&factory);
-    app.read_file(path.c_str());
+
+    std::ifstream ifs(path, std::ios::binary | std::ios::ate);
+    std::streamsize n = ifs.tellg();
+    ifs.seekg(0);
+    std::string content(n, '\0');
+    if (ifs.read(content.data(), n))
+    {
+        app.read_stream(content);
+        doc->recalc_formula_cells();
+    }
 
     return doc;
 }
@@ -70,7 +105,7 @@ class doc_loader
 
 public:
     doc_loader(const pstring& path) :
-        m_doc(), m_factory(m_doc)
+        m_doc({1048576, 16384}), m_factory(m_doc)
     {
         orcus_xls_xml app(&m_factory);
         app.read_file(path.data());
@@ -94,7 +129,8 @@ void update_config(spreadsheet::document& doc, const string& path)
         spreadsheet::document_config cfg = doc.get_config();
 
         yaml::document_tree config;
-        config.load(load_file_content(path.data()));
+        file_content content(path.data());
+        config.load(content.str());
         yaml::const_node root = config.get_document_root(0);
         std::vector<yaml::const_node> keys = root.keys();
         for (size_t i = 0; i < keys.size(); ++i)
@@ -118,6 +154,41 @@ void update_config(spreadsheet::document& doc, const string& path)
 
 void test_xls_xml_import()
 {
+    auto verify = [](spreadsheet::document& doc, const std::string& dir)
+    {
+        std::string path = dir;
+        path.append("config.yaml");
+        update_config(doc, path);
+
+        // Dump the content of the model.
+        ostringstream os;
+        doc.dump_check(os);
+        string check = os.str();
+
+        // Check that against known control.
+        path = dir;
+        path.append("check.txt");
+        file_content control(path.data());
+
+        assert(!check.empty());
+        assert(!control.empty());
+
+        pstring s1(check.data(), check.size());
+        pstring s2 = control.str();
+        s1 = s1.trim();
+        s2 = s2.trim();
+
+        if (s1 != s2)
+        {
+            size_t offset = locate_first_different_char(s1, s2);
+            auto line1 = locate_line_with_offset(s1, offset);
+            auto line2 = locate_line_with_offset(s2, offset);
+            cout << "expected: " << line2.line << endl;
+            cout << "observed: " << line1.line << endl;
+            assert(!"content verification failed");
+        }
+    };
+
     for (const char* dir : dirs)
     {
         cout << dir << endl;
@@ -126,27 +197,11 @@ void test_xls_xml_import()
 
         // Read the input.xml document.
         path.append("input.xml");
-        std::unique_ptr<spreadsheet::document> doc = load_doc(path);
+        std::unique_ptr<spreadsheet::document> doc = load_doc_from_filepath(path);
+        verify(*doc, dir);
 
-        path = dir;
-        path.append("config.yaml");
-        update_config(*doc, path);
-
-        // Dump the content of the model.
-        ostringstream os;
-        doc->dump_check(os);
-        string check = os.str();
-
-        // Check that against known control.
-        path = dir;
-        path.append("check.txt");
-        string control = load_file_content(path.c_str());
-
-        assert(!check.empty());
-        assert(!control.empty());
-
-        pstring s1(&check[0], check.size()), s2(&control[0], control.size());
-        assert(s1.trim() == s2.trim());
+        doc = load_doc_from_stream(path);
+        verify(*doc, dir);
     }
 }
 
@@ -154,7 +209,7 @@ void test_xls_xml_merged_cells()
 {
     const char* filepath = SRCDIR"/test/xls-xml/merged-cells/input.xml";
 
-    std::unique_ptr<spreadsheet::document> doc = load_doc(filepath);
+    std::unique_ptr<spreadsheet::document> doc = load_doc_from_filepath(filepath);
 
     const spreadsheet::sheet* sheet1 = doc->get_sheet("Sheet1");
     assert(sheet1);
@@ -194,7 +249,7 @@ void test_xls_xml_date_time()
 {
     const char* filepath = SRCDIR"/test/xls-xml/date-time/input.xml";
 
-    std::unique_ptr<spreadsheet::document> doc = load_doc(filepath);
+    std::unique_ptr<spreadsheet::document> doc = load_doc_from_filepath(filepath);
 
     const spreadsheet::sheet* sheet1 = doc->get_sheet("Sheet1");
     assert(sheet1);
@@ -226,7 +281,7 @@ void test_xls_xml_date_time()
 void test_xls_xml_bold_and_italic()
 {
     std::unique_ptr<spreadsheet::document> doc =
-        load_doc(SRCDIR"/test/xls-xml/bold-and-italic/input.xml");
+        load_doc_from_filepath(SRCDIR"/test/xls-xml/bold-and-italic/input.xml");
 
     const spreadsheet::sheet* sheet1 = doc->get_sheet("Sheet1");
     assert(sheet1);
@@ -313,7 +368,7 @@ void test_xls_xml_bold_and_italic()
 void test_xls_xml_colored_text()
 {
     std::unique_ptr<spreadsheet::document> doc =
-        load_doc(SRCDIR"/test/xls-xml/colored-text/input.xml");
+        load_doc_from_filepath(SRCDIR"/test/xls-xml/colored-text/input.xml");
 
     const spreadsheet::sheet* sheet1 = doc->get_sheet("ColoredText");
     assert(sheet1);
@@ -413,7 +468,7 @@ void test_xls_xml_column_width_row_height()
     };
 
     std::unique_ptr<spreadsheet::document> doc =
-        load_doc(SRCDIR"/test/xls-xml/column-width-row-height/input.xml");
+        load_doc_from_filepath(SRCDIR"/test/xls-xml/column-width-row-height/input.xml");
 
     const spreadsheet::sheet* sheet1 = doc->get_sheet(0);
     assert(sheet1);
@@ -465,7 +520,7 @@ void test_xls_xml_column_width_row_height()
 void test_xls_xml_background_fill()
 {
     pstring path(SRCDIR"/test/xls-xml/background-color/standard.xml");
-    std::unique_ptr<spreadsheet::document> doc = load_doc(path.str());
+    std::unique_ptr<spreadsheet::document> doc = load_doc_from_filepath(path.str());
 
     spreadsheet::styles& styles = doc->get_styles();
 
@@ -516,10 +571,45 @@ void test_xls_xml_background_fill()
     }
 }
 
+void test_xls_xml_named_colors()
+{
+    std::vector<pstring> paths;
+    paths.emplace_back(SRCDIR"/test/xls-xml/named-colors/input.xml");
+    paths.emplace_back(SRCDIR"/test/xls-xml/named-colors/input-upper.xml");
+
+    for (const pstring& path : paths)
+    {
+        cout << path << endl;
+        std::unique_ptr<spreadsheet::document> doc = load_doc_from_filepath(path.str());
+
+        spreadsheet::styles& styles = doc->get_styles();
+        const ixion::model_context& model = doc->get_model_context();
+
+        spreadsheet::sheet* sh = doc->get_sheet(0);
+        assert(sh);
+
+        for (spreadsheet::row_t row = 1; row < 141; ++row)
+        {
+            // Column B stores the expected RGB value in hex.
+            size_t sid = model.get_string_identifier(ixion::abs_address_t(sh->get_index(), row, 1));
+            const string* s = model.get_string(sid);
+            assert(s);
+            spreadsheet::color_rgb_t expected = spreadsheet::to_color_rgb(*s);
+
+            size_t xf = sh->get_cell_format(row, 0);
+            const spreadsheet::fill_t* fill_data = styles.get_fill(xf);
+            const spreadsheet::color_t& actual = fill_data->fg_color;
+            assert(expected.red == actual.red);
+            assert(expected.green == actual.green);
+            assert(expected.blue == actual.blue);
+        }
+    }
+}
+
 void test_xls_xml_text_alignment()
 {
     pstring path(SRCDIR"/test/xls-xml/text-alignment/input.xml");
-    std::unique_ptr<spreadsheet::document> doc = load_doc(path.str());
+    std::unique_ptr<spreadsheet::document> doc = load_doc_from_filepath(path.str());
 
     spreadsheet::styles& styles = doc->get_styles();
 
@@ -586,7 +676,7 @@ void test_xls_xml_cell_borders_single_cells()
 {
     pstring path(SRCDIR"/test/xls-xml/borders/single-cells.xml");
     cout << path << endl;
-    std::unique_ptr<spreadsheet::document> doc = load_doc(path.str());
+    std::unique_ptr<spreadsheet::document> doc = load_doc_from_filepath(path.str());
 
     spreadsheet::styles& styles = doc->get_styles();
 
@@ -638,7 +728,7 @@ void test_xls_xml_cell_borders_directions()
 {
     pstring path(SRCDIR"/test/xls-xml/borders/directions.xml");
     cout << path << endl;
-    std::unique_ptr<spreadsheet::document> doc = load_doc(path.str());
+    std::unique_ptr<spreadsheet::document> doc = load_doc_from_filepath(path.str());
 
     spreadsheet::styles& styles = doc->get_styles();
 
@@ -751,7 +841,7 @@ void test_xls_xml_cell_borders_colors()
 
     pstring path(SRCDIR"/test/xls-xml/borders/colors.xml");
     cout << path << endl;
-    std::unique_ptr<spreadsheet::document> doc = load_doc(path.str());
+    std::unique_ptr<spreadsheet::document> doc = load_doc_from_filepath(path.str());
 
     spreadsheet::styles& styles = doc->get_styles();
 
@@ -827,7 +917,7 @@ void test_xls_xml_hidden_rows_columns()
 {
     pstring path(SRCDIR"/test/xls-xml/hidden-rows-columns/input.xml");
     cout << path << endl;
-    std::unique_ptr<spreadsheet::document> doc = load_doc(path.str());
+    std::unique_ptr<spreadsheet::document> doc = load_doc_from_filepath(path.str());
 
     spreadsheet::sheet* sh = doc->get_sheet("Hidden Rows");
     assert(sh);
@@ -867,7 +957,7 @@ void test_xls_xml_hidden_rows_columns()
     // The rest of the rows are visible.
     assert(!sh->is_row_hidden(9, &row_start, &row_end));
     assert(row_start == 9);
-    assert(row_end == sh->row_size()); // the end position is non-inclusive.
+    assert(row_end == doc->get_sheet_size().rows); // the end position is non-inclusive.
 
     sh = doc->get_sheet("Hidden Columns");
     assert(sh);
@@ -897,7 +987,7 @@ void test_xls_xml_hidden_rows_columns()
     // The rest of the columns are all visible.
     assert(!sh->is_col_hidden(11, &col_start, &col_end));
     assert(col_start == 11);
-    assert(col_end == sh->col_size()); // non-inclusive
+    assert(col_end == doc->get_sheet_size().columns); // non-inclusive
 }
 
 void test_xls_xml_character_set()
@@ -909,11 +999,64 @@ void test_xls_xml_character_set()
     assert(loader.get_factory().get_character_set() == character_set_t::windows_1252);
 }
 
+void test_xls_xml_number_format()
+{
+    pstring path(SRCDIR"/test/xls-xml/number-format/date-time.xml");
+    doc_loader loader(path);
+
+    const spreadsheet::document& doc = loader.get_doc();
+    const spreadsheet::styles& styles = doc.get_styles();
+
+    const spreadsheet::sheet* sh = doc.get_sheet(0);
+    assert(sh);
+
+    struct check
+    {
+        ss::row_t row;
+        ss::col_t col;
+        pstring expected;
+    };
+
+    std::vector<check> checks =
+    {
+        { 1, 1, "[$-F800]dddd\\,\\ mmmm\\ dd\\,\\ yyyy" },
+        { 2, 1, "[ENG][$-409]mmmm\\ d\\,\\ yyyy;@" },
+        { 3, 1, "m/d/yy;@" },
+        { 4, 1, "m/d/yyyy h:mm" }, // General Date
+        { 5, 1, "d-mmm-yy" }, // Medium Date
+        { 6, 1, "m/d/yyyy" }, // Short Date
+        { 7, 1, "h:mm:ss AM/PM" }, // Long Time
+        { 8, 1, "h:mm AM/PM" }, // Medium Time
+        { 9, 1, "h:mm" }, // Short Time
+        { 10, 1, "0.00" }, // Fixed
+        { 11, 1, "#,##0.00" }, // Standard
+        { 12, 1, "0.00%" }, // Percent
+        { 13, 1, "0.00E+00" }, // Scientific
+        { 14, 1, "\"Yes\";\"Yes\";\"No\"" }, // Yes/No
+        { 15, 1, "\"True\";\"True\";\"False\"" }, // True/False
+        { 16, 1, "\"On\";\"On\";\"Off\"" }, // On/Off
+        { 17, 1, "$#,##0.00_);[Red]($#,##0.00)" }, // Currency
+        { 18, 1, "[$\xe2\x82\xac-x-euro2] #,##0.00_);[Red]([$\xe2\x82\xac-x-euro2] #,##0.00)" }, // Euro Currency
+    };
+
+    for (const check& c : checks)
+    {
+        size_t xf = sh->get_cell_format(c.row, c.col);
+        const spreadsheet::cell_format_t* cf = styles.get_cell_format(xf);
+        assert(cf);
+
+        const spreadsheet::number_format_t* nf = styles.get_number_format(cf->number_format);
+        assert(nf);
+        assert(nf->format_string == c.expected);
+    }
+}
+
 void test_xls_xml_view_cursor_per_sheet()
 {
     string path(SRCDIR"/test/xls-xml/view/cursor-per-sheet.xml");
 
-    spreadsheet::document doc;
+    spreadsheet::range_size_t ss{1048576, 16384};
+    spreadsheet::document doc{ss};
     spreadsheet::view view(doc);
     spreadsheet::import_factory factory(doc, view);
     orcus_xls_xml app(&factory);
@@ -928,11 +1071,12 @@ void test_xls_xml_view_cursor_per_sheet()
     assert(sv);
 
     // NB : the resolver type is set to R1C1 for Excel XML 2003.
-    spreadsheet::iface::import_reference_resolver* resolver = factory.get_reference_resolver();
+    spreadsheet::iface::import_reference_resolver* resolver =
+        factory.get_reference_resolver(spreadsheet::formula_ref_context_t::global);
     assert(resolver);
 
     // On Sheet1, the cursor should be set to C4.
-    spreadsheet::range_t expected = resolver->resolve_range(ORCUS_ASCII("R4C3"));
+    spreadsheet::range_t expected = to_rc_range(resolver->resolve_range("R4C3"));
     spreadsheet::range_t actual = sv->get_selection(spreadsheet::sheet_pane_t::top_left);
     assert(expected == actual);
 
@@ -940,7 +1084,7 @@ void test_xls_xml_view_cursor_per_sheet()
     assert(sv);
 
     // On Sheet2, the cursor should be set to D8.
-    expected = resolver->resolve_range(ORCUS_ASCII("R8C4"));
+    expected = to_rc_range(resolver->resolve_range("R8C4"));
     actual = sv->get_selection(spreadsheet::sheet_pane_t::top_left);
     assert(expected == actual);
 
@@ -948,7 +1092,7 @@ void test_xls_xml_view_cursor_per_sheet()
     assert(sv);
 
     // On Sheet3, the cursor should be set to D2.
-    expected = resolver->resolve_range(ORCUS_ASCII("R2C4"));
+    expected = to_rc_range(resolver->resolve_range("R2C4"));
     actual = sv->get_selection(spreadsheet::sheet_pane_t::top_left);
     assert(expected == actual);
 
@@ -956,7 +1100,7 @@ void test_xls_xml_view_cursor_per_sheet()
     assert(sv);
 
     // On Sheet4, the cursor should be set to C5:E8.
-    expected = resolver->resolve_range(ORCUS_ASCII("R5C3:R8C5"));
+    expected = to_rc_range(resolver->resolve_range("R5C3:R8C5"));
     actual = sv->get_selection(spreadsheet::sheet_pane_t::top_left);
     assert(expected == actual);
 }
@@ -972,7 +1116,8 @@ void test_xls_xml_view_cursor_split_pane()
 {
     string path(SRCDIR"/test/xls-xml/view/cursor-split-pane.xml");
 
-    spreadsheet::document doc;
+    spreadsheet::range_size_t ss{1048576, 16384};
+    spreadsheet::document doc{ss};
     spreadsheet::view view(doc);
     spreadsheet::import_factory factory(doc, view);
     orcus_xls_xml app(&factory);
@@ -981,7 +1126,8 @@ void test_xls_xml_view_cursor_split_pane()
     app.read_file(path.c_str());
 
     // NB : the resolver type is set to R1C1 for Excel XML 2003.
-    spreadsheet::iface::import_reference_resolver* resolver = factory.get_reference_resolver();
+    spreadsheet::iface::import_reference_resolver* resolver =
+        factory.get_reference_resolver(spreadsheet::formula_ref_context_t::global);
     assert(resolver);
 
     // Sheet4 should be active.
@@ -996,7 +1142,7 @@ void test_xls_xml_view_cursor_split_pane()
     assert(sv->get_split_pane().ver_split == 1800.0);
 
     {
-        spreadsheet::address_t expected = resolver->resolve_address(ORCUS_ASCII("R6C6"));
+        spreadsheet::address_t expected = to_rc_address(resolver->resolve_address("R6C6"));
         spreadsheet::address_t actual = sv->get_split_pane().top_left_cell;
         assert(expected == actual);
     }
@@ -1012,7 +1158,7 @@ void test_xls_xml_view_cursor_split_pane()
     for (const expected_selection& es : expected_selections)
     {
         // cursor in the top-left pane.
-        spreadsheet::range_t expected = resolver->resolve_range(es.sel, es.sel_n);
+        spreadsheet::range_t expected = to_rc_range(resolver->resolve_range({es.sel, es.sel_n}));
         spreadsheet::range_t actual = sv->get_selection(es.pane);
         assert(expected == actual);
     }
@@ -1026,7 +1172,7 @@ void test_xls_xml_view_cursor_split_pane()
     assert(sv->get_split_pane().ver_split == 2400.0);
 
     {
-        spreadsheet::address_t expected = resolver->resolve_address(ORCUS_ASCII("R8C6"));
+        spreadsheet::address_t expected = to_rc_address(resolver->resolve_address("R8C6"));
         spreadsheet::address_t actual = sv->get_split_pane().top_left_cell;
         assert(expected == actual);
     }
@@ -1042,7 +1188,7 @@ void test_xls_xml_view_cursor_split_pane()
     for (const expected_selection& es : expected_selections)
     {
         // cursor in the top-left pane.
-        spreadsheet::range_t expected = resolver->resolve_range(es.sel, es.sel_n);
+        spreadsheet::range_t expected = to_rc_range(resolver->resolve_range({es.sel, es.sel_n}));
         spreadsheet::range_t actual = sv->get_selection(es.pane);
         assert(expected == actual);
     }
@@ -1056,7 +1202,7 @@ void test_xls_xml_view_cursor_split_pane()
     assert(sv->get_split_pane().ver_split == 1500.0);
 
     {
-        spreadsheet::address_t expected = resolver->resolve_address(ORCUS_ASCII("R5C1"));
+        spreadsheet::address_t expected = to_rc_address(resolver->resolve_address("R5C1"));
         spreadsheet::address_t actual = sv->get_split_pane().top_left_cell;
         assert(expected == actual);
     }
@@ -1070,7 +1216,7 @@ void test_xls_xml_view_cursor_split_pane()
     for (const expected_selection& es : expected_selections)
     {
         // cursor in the top-left pane.
-        spreadsheet::range_t expected = resolver->resolve_range(es.sel, es.sel_n);
+        spreadsheet::range_t expected = to_rc_range(resolver->resolve_range({es.sel, es.sel_n}));
         spreadsheet::range_t actual = sv->get_selection(es.pane);
         assert(expected == actual);
     }
@@ -1084,7 +1230,7 @@ void test_xls_xml_view_cursor_split_pane()
     assert(sv->get_split_pane().ver_split == 0.0);
 
     {
-        spreadsheet::address_t expected = resolver->resolve_address(ORCUS_ASCII("R1C5"));
+        spreadsheet::address_t expected = to_rc_address(resolver->resolve_address("R1C5"));
         spreadsheet::address_t actual = sv->get_split_pane().top_left_cell;
         assert(expected == actual);
     }
@@ -1098,7 +1244,7 @@ void test_xls_xml_view_cursor_split_pane()
     for (const expected_selection& es : expected_selections)
     {
         // cursor in the top-left pane.
-        spreadsheet::range_t expected = resolver->resolve_range(es.sel, es.sel_n);
+        spreadsheet::range_t expected = to_rc_range(resolver->resolve_range({es.sel, es.sel_n}));
         spreadsheet::range_t actual = sv->get_selection(es.pane);
         assert(expected == actual);
     }
@@ -1108,7 +1254,8 @@ void test_xls_xml_view_frozen_pane()
 {
     string path(SRCDIR"/test/xls-xml/view/frozen-pane.xml");
 
-    spreadsheet::document doc;
+    spreadsheet::range_size_t ss{1048576, 16384};
+    spreadsheet::document doc{ss};
     spreadsheet::view view(doc);
     spreadsheet::import_factory factory(doc, view);
     orcus_xls_xml app(&factory);
@@ -1117,7 +1264,8 @@ void test_xls_xml_view_frozen_pane()
     app.read_file(path.c_str());
 
     // NB : the resolver type is set to R1C1 for Excel XML 2003.
-    spreadsheet::iface::import_reference_resolver* resolver = factory.get_reference_resolver();
+    spreadsheet::iface::import_reference_resolver* resolver =
+        factory.get_reference_resolver(spreadsheet::formula_ref_context_t::global);
     assert(resolver);
 
     // Sheet3 should be active.
@@ -1129,7 +1277,7 @@ void test_xls_xml_view_frozen_pane()
     {
         // Sheet1 is vertically frozen between columns A and B.
         const spreadsheet::frozen_pane_t& fp = sv->get_frozen_pane();
-        assert(fp.top_left_cell == resolver->resolve_address(ORCUS_ASCII("R1C2")));
+        assert(fp.top_left_cell == to_rc_address(resolver->resolve_address("R1C2")));
         assert(fp.visible_columns == 1);
         assert(fp.visible_rows == 0);
         assert(sv->get_active_pane() == spreadsheet::sheet_pane_t::top_right);
@@ -1141,7 +1289,7 @@ void test_xls_xml_view_frozen_pane()
     {
         // Sheet2 is horizontally frozen between rows 1 and 2.
         const spreadsheet::frozen_pane_t& fp = sv->get_frozen_pane();
-        assert(fp.top_left_cell == resolver->resolve_address(ORCUS_ASCII("R2C1")));
+        assert(fp.top_left_cell == to_rc_address(resolver->resolve_address("R2C1")));
         assert(fp.visible_columns == 0);
         assert(fp.visible_rows == 1);
         assert(sv->get_active_pane() == spreadsheet::sheet_pane_t::bottom_left);
@@ -1153,13 +1301,49 @@ void test_xls_xml_view_frozen_pane()
     {
         // Sheet3 is frozen both horizontally and vertically.
         const spreadsheet::frozen_pane_t& fp = sv->get_frozen_pane();
-        assert(fp.top_left_cell == resolver->resolve_address(ORCUS_ASCII("R9C5")));
+        assert(fp.top_left_cell == to_rc_address(resolver->resolve_address("R9C5")));
         assert(fp.visible_columns == 4);
         assert(fp.visible_rows == 8);
         assert(sv->get_active_pane() == spreadsheet::sheet_pane_t::bottom_right);
     }
 }
 
+void test_xls_xml_skip_error_cells()
+{
+    string path(SRCDIR"/test/xls-xml/formula-cells-parse-error/input.xml");
+
+    try
+    {
+        auto doc = load_doc_from_filepath(path, false, ss::formula_error_policy_t::fail);
+        (void)doc;
+        assert(!"exception was expected, but was not thrown.");
+    }
+    catch (const std::exception& e)
+    {
+        // works as expected
+    }
+
+    auto doc = load_doc_from_filepath(path, false, ss::formula_error_policy_t::skip);
+    const ixion::model_context& cxt = doc->get_model_context();
+
+    auto is_formula_cell_with_error = [&cxt](const ixion::abs_address_t& pos) -> bool
+    {
+        const ixion::formula_cell* fc = cxt.get_formula_cell(pos);
+        if (!fc)
+            return false;
+
+        const ixion::formula_tokens_t& tokens = fc->get_tokens()->get();
+        if (tokens.empty())
+            return false;
+
+        return tokens[0]->get_opcode() == ixion::fop_error;
+    };
+
+    // Make sure these two cells have been imported as formula cells with error.
+    assert(is_formula_cell_with_error(ixion::abs_address_t(0, 1, 1)));
+    assert(is_formula_cell_with_error(ixion::abs_address_t(0, 4, 0)));
+}
+
 }
 
 int main()
@@ -1174,18 +1358,22 @@ int main()
     test_xls_xml_colored_text();
     test_xls_xml_column_width_row_height();
     test_xls_xml_background_fill();
+    test_xls_xml_named_colors();
     test_xls_xml_text_alignment();
     test_xls_xml_cell_borders_single_cells();
     test_xls_xml_cell_borders_directions();
     test_xls_xml_cell_borders_colors();
     test_xls_xml_hidden_rows_columns();
     test_xls_xml_character_set();
+    test_xls_xml_number_format();
 
     // view import
     test_xls_xml_view_cursor_per_sheet();
     test_xls_xml_view_cursor_split_pane();
     test_xls_xml_view_frozen_pane();
 
+    test_xls_xml_skip_error_cells();
+
     return EXIT_SUCCESS;
 }
 
