@@ -12,9 +12,11 @@
 
 #include "orcus_filter_global.hpp"
 
+#include <iostream>
+
 using namespace orcus;
 
-int main(int argc, char** argv)
+int main(int argc, char** argv) try
 {
     spreadsheet::range_size_t ss{1048576, 16384};
     spreadsheet::document doc{ss};
@@ -26,4 +28,11 @@ int main(int argc, char** argv)
         return EXIT_FAILURE;
 
     return EXIT_SUCCESS;
-}/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
+}
+catch (const std::exception& e)
+{
+    std::cerr << e.what() << std::endl;
+    return EXIT_FAILURE;
+}
+
+/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
