@@ -72,7 +72,7 @@ std::string parse_error::build_message(
     if (msg_before)
         os << msg_before;
 
-    write_to(os, p, n);
+    os << std::string_view(p, n);
 
     if (msg_after)
         os << msg_after;
@@ -130,7 +130,7 @@ void parser_base::skip(const char* chars_to_skip, size_t n_chars_to_skip)
 #else
     for (; has_char(); next())
     {
-        if (!is_in(*mp_char, chars_to_skip, n_chars_to_skip))
+        if (!is_in(*mp_char, {chars_to_skip, n_chars_to_skip}))
             break;
     }
 #endif
