@@ -5,10 +5,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include "orcus/tokens.hpp"
-#include "orcus/pstring.hpp"
-
-using namespace std;
+#include <orcus/tokens.hpp>
 
 namespace orcus {
 
@@ -17,11 +14,7 @@ tokens::tokens(const char** token_names, size_t token_name_count) :
     m_token_name_count(token_name_count)
 {
     for (size_t i = 0; i < m_token_name_count; ++i)
-    {
-        m_tokens.insert(
-            token_map_type::value_type(
-                pstring(m_token_names[i]), static_cast<xml_token_t>(i)));
-    }
+        m_tokens.emplace(m_token_names[i], xml_token_t(i));
 }
 
 bool tokens::is_valid_token(xml_token_t token) const
@@ -29,7 +22,7 @@ bool tokens::is_valid_token(xml_token_t token) const
     return token != XML_UNKNOWN_TOKEN;
 }
 
-xml_token_t tokens::get_token(const pstring& name) const
+xml_token_t tokens::get_token(std::string_view name) const
 {
     token_map_type::const_iterator itr = m_tokens.find(name);
     if (itr == m_tokens.end())
@@ -37,9 +30,9 @@ xml_token_t tokens::get_token(const pstring& name) const
     return itr->second;
 }
 
-const char* tokens::get_token_name(xml_token_t token) const
+std::string_view tokens::get_token_name(xml_token_t token) const
 {
-    if (static_cast<size_t>(token) >= m_token_name_count)
+    if (size_t(token) >= m_token_name_count)
         return "";
 
     return m_token_names[token];
