@@ -19,6 +19,7 @@
 #include <ixion/model_context.hpp>
 #include <ixion/formula.hpp>
 #include <ixion/formula_result.hpp>
+#include <ixion/cell.hpp>
 
 #include <sstream>
 
@@ -69,7 +70,7 @@ const char* css_style_global =
 "}\n"
 
 "td { "
-    "width : 1in; "
+    "width : 1in; border: 1px solid lightgray; "
 "}\n"
 
 "td.empty { "
@@ -158,7 +159,11 @@ void print_formatted_text(std::ostream& strm, const std::string& text, const for
             style += "font-style: normal;";
 
         if (!run.font.empty())
-            style += "font-family: " + run.font.str() + ";";
+        {
+            style += "font-family: ";
+            style += run.font;
+            style += ";";
+        }
 
         if (run.font_size)
         {
@@ -434,7 +439,8 @@ void html_dumper::dump(std::ostream& os) const
             return;
 
         const ixion::model_context& cxt = m_doc.get_model_context();
-        const ixion::formula_name_resolver* resolver = m_doc.get_formula_name_resolver();
+        const ixion::formula_name_resolver* resolver =
+            m_doc.get_formula_name_resolver(spreadsheet::formula_ref_context_t::global);
         const import_shared_strings* sstrings = m_doc.get_shared_strings();
 
         elem table(os, p_table);
@@ -472,9 +478,9 @@ void html_dumper::dump(std::ostream& os) const
                 if (!p_merge_size && p_overlapped)
                 {
                     // Check if this cell is overlapped by a merged cell.
-                    bool overlapped = false;
-                    col_t last_col;
-                    if (p_overlapped->search_tree(col, overlapped, nullptr, &last_col).second && overlapped)
+                    col_t overlapped_origin = -1;
+                    col_t last_col = -1;
+                    if (p_overlapped->search_tree(col, overlapped_origin, nullptr, &last_col).second && overlapped_origin >= 0)
                     {
                         // Skip all overlapped cells on this row.
                         col = last_col - 1;
@@ -571,8 +577,16 @@ void html_dumper::dump(std::ostream& os) const
                             else
                                 os << formula;
 
-                            ixion::formula_result res = cell->get_result_cache();
-                            os << " (" << res.str(m_doc.get_model_context()) << ")";
+                            try
+                            {
+                                ixion::formula_result res = cell->get_result_cache(
+                                    ixion::formula_result_wait_policy_t::throw_exception);
+                                os << " (" << res.str(m_doc.get_model_context()) << ")";
+                            }
+                            catch (const std::exception&)
+                            {
+                                os << " (#RES!)";
+                            }
                         }
 
                         break;
@@ -614,6 +628,8 @@ void html_dumper::build_overlapped_ranges()
     if (!sh)
         return;
 
+    range_size_t sheet_size = m_doc.get_sheet_size();
+
     detail::col_merge_size_type::const_iterator it_col = m_merge_ranges.begin(), it_col_end = m_merge_ranges.end();
     for (; it_col != it_col_end; ++it_col)
     {
@@ -630,7 +646,7 @@ void html_dumper::build_overlapped_ranges()
                 detail::overlapped_cells_type::iterator it_cont = m_overlapped_ranges.find(row);
                 if (it_cont == m_overlapped_ranges.end())
                 {
-                    auto p = orcus::make_unique<detail::overlapped_col_index_type>(0, sh->col_size(), false);
+                    auto p = std::make_unique<detail::overlapped_col_index_type>(0, sheet_size.columns, -1);
                     std::pair<detail::overlapped_cells_type::iterator, bool> r =
                         m_overlapped_ranges.insert(detail::overlapped_cells_type::value_type(row, std::move(p)));
 
@@ -644,7 +660,7 @@ void html_dumper::build_overlapped_ranges()
                 }
 
                 detail::overlapped_col_index_type& cont = *it_cont->second;
-                cont.insert_back(col, col+item.width, true);
+                cont.insert_back(col, col+item.width, col);
             }
         }
     }
